import { AuthInfo, LoginResult } from '../auth/auth.types';
/**
 * General Message types
 * @public
 */
export declare enum MessageType {
    LOGON_ACK = "LOGON_ACK",
    LOGON_NACK = "LOGON_NACK",
    LOGOFF_ACK = "LOGOFF_ACK",
    LOGOFF_NACK = "LOGOFF_NACK",
    HEARTBEAT_PING = "HEARTBEAT_PING",
    HEARTBEAT_PONG = "HEARTBEAT_PONG",
    MORE_ROWS = "MORE_ROWS",
    MSG_ACK = "MSG_ACK",
    MSG_NACK = "MSG_NACK",
    LOGOUT_ACK = "LOGOUT_ACK",
    LOGOUT_NACK = "LOGOUT_NACK",
    QUERY_UPDATE = "QUERY_UPDATE"
}
/**
 * Event Message types
 * @public
 */
export declare enum EventMessageType {
    EVENT_LOGIN_AUTH = "EVENT_LOGIN_AUTH",
    EVENT_LOGIN_AUTH_ACK = "EVENT_LOGIN_AUTH_ACK",
    EVENT_LOGIN_AUTH_NACK = "EVENT_LOGIN_AUTH_NACK",
    EVENT_LOGOUT = "EVENT_LOGOUT",
    DATA_LOGON = "DATA_LOGON",
    DATA_LOGOFF = "DATA_LOGOFF",
    META_REQUEST = "META_REQUEST",
    JSON_SCHEMA_REQUEST = "JSON_SCHEMA_REQUEST",
    RESOURCES_REQUEST = "RESOURCES_REQUEST",
    EVENT_CHANGE_USER_PASSWORD = "EVENT_CHANGE_USER_PASSWORD",
    EVENT_SELF_SERVICE_PASSWORD_RESET = "EVENT_SELF_SERVICE_PASSWORD_RESET",
    EVENT_PASSWORD_RESET_ACTION = "EVENT_PASSWORD_RESET_ACTION"
}
/**
 * General Message codes
 * @public
 */
export declare enum MessageCode {
    SESSION_NOT_FOUND = "SESSION_NOT_FOUND",
    UNKNOWN_MESSAGE_TYPE = "UNKNOWN_MESSAGE_TYPE"
}
/** Namespace for all Message detail types
 * @public
 */
export declare namespace MessageDetails {
    /**
     * Details type for a basic auth message
     * @public
     */
    type LoginBasic = {
        USER_NAME: string;
        PASSWORD: string;
    };
    /**
     * Details type for a SSO-based auth message
     * @public
     */
    type LoginSSO = {
        SSO_TOKEN: string;
    };
    /**
     * Details type for a JWT-based auth message
     * @public
     */
    type LoginJWT = {
        JWT: string;
        DOMAIN: string;
    };
    /**
     * Union type for different types of login information
     * @public
     */
    type Login = LoginBasic | LoginSSO | LoginJWT;
    /**
     * Details type for a password-change message
     * @public
     */
    type ChangePassword = {
        OLD_PASSWORD: string;
        NEW_PASSWORD: string;
    };
    /**
     * Details type for a password-reset message
     * @public
     */
    type ResetPassword = {
        USER_NAME: string;
    };
    /**
     * Details type for a forgot-password message
     * @public
     */
    type ForgotPassword = {
        USER_NAME: string;
        RETURN_URL: string;
    };
    /**
     * Details type for a forgot-password token-based message
     * @public
     */
    type ForgotPasswordToken = {
        USER_NAME: string;
        RESET_TOKEN: string;
        NEW_PASSWORD: string;
    };
    /**
     * Details type for a password-expiry message
     * @public
     */
    type ExpirePassword = {
        USER_NAME: string;
        PASSWORD: string;
    };
    /**
     * Details type for a refresh-token message
     * @public
     */
    type RefreshToken = {
        REFRESH_AUTH_TOKEN: string;
        MFA_CODE?: string;
    };
    /**
     * Details type to request more rows
     * @public
     */
    type MoreRows = {
        SOURCE_REF: string;
    };
    /**
     * Details type to update columns
     * @public
     */
    type UpdateColumns = {
        SOURCE_REF: string;
        ADD_COLUMNS: string;
        DROP_COLUMNS: string;
    };
    /**
     * Details type to request DATASERVER metadata
     * @public
     */
    type DataserverRequest = {
        DATASOURCE_NAME: string;
    };
    /**
     * Details type to request FEATURE metadata
     * @public
     */
    type MetaRequest = {
        FEATURE: string;
    };
    /**
     * Details type for ERROR results
     * @public
     */
    type Error = {
        ERROR: string;
    };
    /**
     * System-level information when a user is logged in.
     * @public
     */
    type LoggedInSystemInfo = {
        DAYS_TO_PASSWORD_EXPIRY: number;
        FAILED_LOGIN_ATTEMPTS: number;
        HEARTBEAT_INTERVAL_SECS: number;
        LAST_LOGIN_DATETIME: number;
        NOTIFY_EXPIRY?: boolean;
        PRODUCT?: any[];
        SYSTEM?: any;
    };
}
/**
 * Error definition for a message result
 * @public
 */
export type MessageError = {
    CODE: MessageCode | string;
    STATUS_CODE?: string;
    TEXT: string;
};
/**
 * Base Message type for all DETAIL variants
 * @public
 */
export type Message<T = void | any> = {
    REPLY?: any[];
    CODE?: MessageCode | string;
    DETAILS?: T;
    LOGGED_OUT?: boolean;
    ERROR?: MessageError[];
    MESSAGE_TYPE?: MessageType | EventMessageType | string;
    PERMISSION?: string[];
    PROFILE?: string[];
    RESOURCES?: any[];
    REFRESH_AUTH_TOKEN?: string;
    ROW?: any[];
    ROWS_COUNT?: number;
    SEQUENCE_ID?: number;
    SERVICE_NAME?: string;
    SESSION_AUTH_TOKEN?: string;
    SESSION_ID?: string;
    SOURCE_REF?: string;
    USER_NAME?: string;
    WARNING?: any[];
};
/**
 * Parameters for a REQUEST message
 * @public
 */
export type RequestParams = {
    DETAILS?: any;
    REQUEST?: any;
};
/**
 * Parameters for a COMMIT message
 * @public
 */
export type CommitParams = {
    DETAILS: any;
    HOSTNAME?: string;
    IGNORE_WARNINGS?: boolean;
    VALIDATE?: boolean;
};
/**
 * Check if the provided message contains an error and throw an Error if it does
 * @public
 * @param message - Message to check for errors
 * @returns The message if no errors are present, otherwise throws an Error
 */
export declare const messageOrThrow: (message: Message) => Message<any>;
/**
 * A utility class for building different types of messages.
 * @public
 */
export interface MessageBuilder {
    /**
     * Creates a login message.
     *
     * @remarks EVENT_LOGIN_AUTH
     * @public
     * @param credentials - The authentication credentials.
     * @returns The login message.
     */
    createLoginMessage(credentials: AuthInfo): Message<MessageDetails.Login>;
    /**
     * Creates a logout message.
     *
     * @remarks EVENT_LOGOUT
     * @public
     * @param loginResult - The login result.
     * @returns The logout message.
     */
    createLogoutMessage(loginResult?: LoginResult): Message;
    /**
     * Creates a message to logon to a DATASERVER.
     * Used for data fetching.
     *
     * @remarks DATA_LOGON
     * @public
     * @param resourceName - The resource name.
     * @param params - The additional parameters.
     * @returns The data logon message.
     */
    createDataLogonMessage(resourceName: string, params?: any): Message<MessageDetails.DataserverRequest>;
    /**
     * Creates a message to logoff from a DATASERVER.
     *
     * @remarks DATA_LOGOFF
     * @public
     * @param sourceRef - The source reference.
     * @returns The data logoff message.
     */
    createDataLogoffMessage<T = any>(sourceRef: string): Message<T>;
    /**
     * Creates a message to request more rows.
     *
     * @remarks MORE_ROWS
     * @public
     * @param sourceRef - The source reference.
     * @returns The more rows message.
     */
    createMoreRowsMessage(sourceRef: string): Message;
    /**
     * Creates a request message.
     *
     * @remarks
     * For REQUEST_SERVER resources. You can send with REQ_ or without it.
     * @public
     * @param resourceName - The resource name.
     * @param params - The additional parameters.
     * @returns The request message.
     */
    createRequestMessage<T = any>(resourceName: string, params?: any): Message<T>;
    /**
     * Creates a message for getting list of resources.
     *
     * @public
     * @param params - The additional parameters.
     * @returns The request message.
     */
    createResourcesMessage<T = any>(params?: any): Message<T>;
    /**
     * Creates a metadata fetch message.
     *
     * @remarks META_REQUEST
     * @public
     * @param resourceName - The resource name.
     * @param messageType - The event message type.
     * @returns The meta request message.
     */
    createMetaRequestMessage(resourceName: string, messageType?: EventMessageType): Message<MessageDetails.MetaRequest>;
    /**
     * Creates an event commit message.
     *
     * @public
     * @param eventName - The event name.
     * @param params - The additional parameters.
     * @returns The commit message.
     */
    createCommitMessage<T = any>(eventName: string, params?: CommitParams): Message<T>;
    /**
     * Creates an auth message specific for token refresh.
     *
     * @remarks EVENT_LOGIN_AUTH
     * @public
     * @returns The refresh token message.
     */
    createRefreshTokenMessage(): Message<MessageDetails.RefreshToken>;
    /**
     * Creates a change password message.
     *
     * @remarks EVENT_CHANGE_USER_PASSWORD
     * @public
     * @param username - The username.
     * @param oldPassword - The old password.
     * @param newPassword - The new password.
     * @returns The change password message.
     */
    createChangePasswordMessage(username: string, oldPassword: string, newPassword: string): Message<MessageDetails.ChangePassword>;
    /**
     * Creates a forgot password message.
     *
     * @remarks EVENT_SELF_SERVICE_PASSWORD_RESET
     * @public
     * @param username - The username.
     * @param returnUrl - The return URL.
     * @param requester - The requester.
     * @returns The forgot password message.
     */
    createForgotPasswordMessage(username: string, returnUrl: string, requester?: string): Message<MessageDetails.ForgotPassword>;
    /**
     * Creates a forgot password token message.
     *
     * @public
     * @param username - The username.
     * @param resetToken - The reset token.
     * @param newPassword - The new password.
     * @param requester - The requester.
     * @returns The forgot password token message.
     */
    createForgotPasswordTokenMessage(username: string, resetToken: string, newPassword: string, requester?: string): Message<MessageDetails.ForgotPasswordToken>;
    /**
     * Create HTTP Headers from a message.
     *
     * @public
     * @param message - The message.
     * @param contentType - Optional content-type, defaults to `application/json; charset=UTF-8`.
     * @returns The headers.
     */
    createHTTPHeadersFromMessage(message: Message, contentType?: string): HeadersInit;
}
/**
 * The DI token for the MessageBUilder interface.
 * @internal
 */
export declare const MessageBuilder: import("@microsoft/fast-foundation").InterfaceSymbol<MessageBuilder>;
//# sourceMappingURL=message.d.ts.map