import { BehaviorSubject, Observable, Observer, Subject, Subscription } from 'rxjs';
import { Message } from './message';
import { SocketReconnectOptions } from './reconnectStrategy';
/**
 * Subject that wraps a WebSocketSubject and provides a Subject interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Subject.
 */
export declare class SocketSubject<T> extends Subject<T> {
    constructor();
}
/**
 * Observable that wraps a WebSocketSubject and provides an Observable interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Observable.
 */
export declare class SocketObservable<T> extends Observable<T> {
    constructor(...args: any[]);
}
/**
 * An alias to RxJS Subscription.
 * @public
 */
export { Subscription as SocketSubscription };
/**
 * The Socket connection options definition.
 * @public
 */
export type SocketConnectOptions = {
    /**
     * Experimental flag to force reconnection all error scenarios (1011, 1012, etc). Won't rely on reconnectAttempts value.
     * @experimental
     * @remarks
     * This will force the socket to reconnect if the connection is lost even on "final" errors. Only disconnects when directly calling reset()
     * @see Socket#reset
     */
    alwaysOn?: boolean;
    /**
     * An observer to receive notifications of incoming messages from the server.
     */
    connectObserver?: Observer<Message>;
    /**
     * Enable sending heartbeat messages to the server to prevent the connection from timing out.
     */
    enableHeartbeat?: boolean;
    /**
     * Enable logging of heartbeat messages.
     */
    enableHeartbeatLogging?: boolean;
    /**
     * Enable logging of incoming and outgoing WS messages.
     */
    enableMessageLogging?: boolean;
    /**
     * Interval between heartbeat messages
     * @remarks
     * Time in milliseconds
     */
    heartbeatInterval?: number;
    /**
     * The path to a proxy server to use for the connection.
     */
    proxyPath?: string;
};
/**
 * The Socket message handler definition.
 * @public
 */
export declare class SocketMessageHandler {
    constructor(message: Message, onMessage: Function, onError: Function, isSubscription: boolean);
    /**
     * The message that was sent.
     * @public
     */
    get sentMessage(): Message;
    /**
     * The handler function for incoming messages.
     * @public
     */
    get onReceivedMessage(): Function;
    /**
     * The handler function for errors.
     * @public
     */
    get onReceivedError(): Function;
    /**
     * Whether the message handler is a subscription.
     * @public
     */
    get isSubscription(): boolean;
    /**
     * @internal
     */
    private _message;
    /**
     * @internal
     */
    private _onMessage;
    /**
     * @internal
     */
    private _onError;
    /**
     * @internal
     */
    private _isSubscription;
}
/**
 * Represents a WebSocket used to establish a communication channel between client and a Genesis server.
 * Provides methods for socket connection, sending and receiving WS messages.
 * @public
 */
export interface Socket {
    isConnected: boolean;
    isReconnecting: boolean;
    hasValidSession: boolean;
    isConnectedSubject: BehaviorSubject<boolean>;
    connect(host: string, options?: SocketConnectOptions, reconnectOptions?: SocketReconnectOptions): Promise<boolean>;
    /**
     * Disconnects the socket and clears all subscriptions
     */
    reset(): void;
    send<T>(message: Message<T | any>, needsHandling?: boolean): Promise<Message | any>;
    sendForStream<T>(message: Message<T | any>, onMessage: Function, onError: Function): Observable<Message>;
    socketMessages(): SocketSubject<Message>;
}
/**
 * The DI token for the Socket interface.
 * @internal
 */
export declare const Socket: import("@microsoft/fast-foundation").InterfaceSymbol<Socket>;
//# sourceMappingURL=socket.d.ts.map