/// <reference types="webappsec-credential-management" />

import { AppIntent } from '@finos/fdc3';
import { BehaviorSubject } from 'rxjs';
import { Channel } from '@finos/fdc3';
import { Context } from '@finos/fdc3';
import { ContextHandler } from '@finos/fdc3';
import type { DTOMapper } from '@genesislcap/foundation-utils';
import { IntentResolution } from '@finos/fdc3';
import { Intents } from '@finos/fdc3';
import { InterfaceSymbol } from '@microsoft/fast-foundation';
import { JSONSchema7 } from 'json-schema';
import { JSONSerializer } from '@genesislcap/foundation-utils';
import { Logger } from '@genesislcap/foundation-logger';
import { Observable } from 'rxjs';
import { Observer } from 'rxjs';
import { Subscription as SocketSubscription } from 'rxjs';
import { Subject } from 'rxjs';

/**
 * Auth is the main interface for authenticating with a Genesis Server endpoint.
 * @public
 */
export declare interface Auth {
    /**
     * Indicates whether Auth is working or is idle.
     * @public
     */
    isWorking: boolean;
    /** This property is intended to be used internally by the authentication system.
     * @internal
     * @remarks
     * TODO: For now, we are keeping `loggedUserResult` until all app and mf usages are reviewed.
     */
    readonly loggedUserResult: LoginResult;
    /**
     * A boolean value indicating whether or not the user is currently logged in.
     * @public
     */
    readonly isLoggedIn: boolean;
    /**
     * An Observable that emits a boolean value indicating whether or not the user is currently logged in.
     * @public
     */
    readonly isLoggedIn$?: Observable<boolean>;
    /**
     * The currently logged-in user, if any.
     * @internal
     */
    readonly currentUser: User;
    /**
     * Logs in the user with the specified credentials.
     * @public
     * @param credentials - The user's login credentials.
     * @returns A Promise that resolves to the result of the login attempt.
     * @see AuthInfo
     */
    login(credentials: AuthInfo): Promise<LoginResult>;
    /**
     * Logs out the current user, if any.
     * @public
     * @returns A Promise that resolves to the result of the logout attempt.
     * @see LogoutResult
     */
    logout(): Promise<LogoutResult>;
    /**
     * Attempts to re-authenticate the user using the existing session data.
     * @public
     * @returns A Promise that resolves to `true` if the user was successfully re-authenticated,
     * or `false` if re-authentication failed.
     */
    reAuthFromSession(): Promise<boolean>;
}

/**
 * The DI token for the Auth interface.
 * @internal
 */
export declare const Auth: InterfaceSymbol<Auth>;

/**
 * Represents an error that occurs during authentication.
 * @public
 */
export declare class AuthError extends Error {
    sent: Message;
    received?: Message;
    constructor(sent: Message, received?: Message, message?: string);
    isIgnorableError(): boolean;
    getMessageError(): string;
    getDetailsError(): string;
    hasPasswordExpiredError(): boolean;
    hasMFAError(): boolean;
    hasErrorCode(code: string): boolean;
    private errorDetailFormatter;
}

/**
 * Union type of all authentication information types.
 * @public
 */
export declare type AuthInfo = BasicAuthInfo | SsoAuthInfo | JwtAuthInfo | RefreshAuthInfo;

/**
 * @internal
 */
export declare class AuthMock implements Auth {
    isWorking: boolean;
    loggedUserResult: LoginResult;
    currentUser: User;
    isLoggedIn: boolean;
    nextLoginResult: LoginResult;
    nextLogoutResult: LogoutResult;
    nextReAuthResult: boolean;
    login(credentials: AuthInfo): Promise<LoginResult>;
    logout(): Promise<LogoutResult>;
    reAuthFromSession(): Promise<boolean>;
}

/**
 * The available ypes of authentication.
 * @public
 */
export declare enum AuthType {
    BASIC = "BASIC",
    SSO = "SSO",
    JWT = "JWT",
    REFRESH = "REFRESH"
}

/**
 * Represents basic authentication information.
 * @public
 */
export declare type BasicAuthInfo = {
    type: AuthType & AuthType.BASIC;
    username: string;
    password: string;
    mfaToken?: string;
};

/**
 * Represents basic credentials for a user.
 * @public
 */
export declare type BasicCredentialsInput = {
    username: string;
    password: string;
};

/**
 * Parameters for a COMMIT message
 * @public
 */
export declare type CommitParams = {
    DETAILS: any;
    HOSTNAME?: string;
    IGNORE_WARNINGS?: boolean;
    VALIDATE?: boolean;
};

/**
 * Represents a connection and its capabilities on a remote server
 * Provides methods to send and receive messages, as well as metadata and schema information.
 * @public
 */
export declare interface Connect {
    /**
     * Indicates whether Connect is working or is idle.
     * @public
     */
    isWorking: boolean;
    /**
     * Indicates whether the connection is currently established.
     * @public
     */
    readonly isConnected: boolean;
    /**
     * An Observable that emits a boolean value indicating whether the connection is currently established.
     * @public
     */
    readonly isConnected$?: Observable<boolean>;
    /**
     * A behavior subject holding the current connection status.
     * @deprecated - Use `isConnected$` instead.
     * @public
     */
    readonly isConnectedSubject?: BehaviorSubject<boolean>;
    /**
     * Sends a message to the server and returns a promise that resolves with the server's response.
     * @internal
     * @param message - The message to send.
     * @typeParam T - The type of the Message's DETAILS payload.
     * @param needsHandling - Whether the response needs to be handled manually. Default: false.
     * @returns A promise that resolves with the server's response.
     */
    send<T>(message: Message<T | any>, needsHandling?: boolean): Promise<Message | any>;
    /**
     * Returns whether the connection is in HTTP mode.
     * @internal
     * @returns True if the connection is in HTTP mode; false otherwise.
     */
    httpMode(): boolean;
    /**
     * Connects to a server at the specified host.
     * @public
     * @param host - The host to connect to.
     * @returns A promise that resolves with a boolean indicating whether the connection was successful.
     */
    connect(host: string): Promise<boolean>;
    /**
     * Disconnects from the server.
     * @public
     */
    disconnect(): void;
    /**
     * Sets the validity of the current session.
     * Use in Auth scenarios where session expires or renews, but the connection is still active.
     * @internal
     * @param valid - Whether the session is valid.
     */
    setValidSession(valid: boolean): void;
    /**
     * Retrieves a snapshot of data for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the snapshot for.
     * @param params - Additional parameters to pass to the server.
     * @returns A promise that resolves with the snapshot data.
     */
    snapshot(resourceName: string, params?: any): Promise<Message>;
    /**
     * Starts listening for updates on the specified resource.
     * @public
     * @param resourceName - The name of the resource to listen for updates on.
     * @param onMessage - A callback function to handle received messages.
     * @param onError - A callback function to handle errors.
     * @param params - Additional parameters to pass to the server.
     * @returns An observable that emits messages received from the server.
     */
    stream(resourceName: string, onMessage: Function, onError: Function, params?: any): SocketObservable<Message>;
    /**
     * Requests additional rows of data for the specified source reference.
     * @public
     * @param sourceRef - The source reference to request more rows for.
     * @returns A promise that resolves with the additional rows of data.
     */
    getMoreRows(sourceRef: string): Promise<Message>;
    /**
     * Logs off from the specified data source.
     * @public
     * @param sourceRef - The reference of the data source to log off from.
     * @returns A promise that resolves with a message indicating success or failure.
     */
    dataLogoff(sourceRef: string): Promise<Message>;
    /**
     * Retrieves the metadata for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the metadata for.
     * @param useCache - Whether to use the cached metadata if available. Default: true.
     * @returns A promise that resolves with the metadata.
     */
    getMetadata(resourceName: string, useCache?: boolean): Promise<Metadata>;
    /**
     * Retrieves the JSON schema for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the schema for.
     */
    getJSONSchema(resourceName: string): Promise<SchemaResponse>;
    /**
     * Sends a request to the server to retrieve data or perform an action.
     * @public
     * @param resourceName - The name of the resource to request.
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response.
     */
    request(resourceName: string, params?: RequestParams): Promise<Message>;
    /**
     * Sends a request to the server to get available resources.
     * @public
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response with available resources.
     */
    getAvailableResources(params?: RequestParams): Promise<Message>;
    /**
     * Sends a request to the server to retrieve data or perform an action.
     * @public
     * @param resourceName - The name of the resource to request.
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response.
     */
    commitEvent(eventName: string, params?: CommitParams): Promise<Message>;
}

/**
 * The DI token for the Connect interface (HTTP | WS).
 * @internal
 */
export declare const Connect: InterfaceSymbol<Connect>;

/**
 * Configuration options for the Connect (WS) instance.
 * @public
 */
export declare interface ConnectConfig {
    /**
     * Options for the initial connection.
     */
    connect: SocketConnectOptions;
    /**
     * Options for automatic reconnection on connection loss.
     */
    reconnect: SocketReconnectOptions;
}

/**
 * Configuration options for the Connect (WS) instance.
 * @internal
 */
export declare const ConnectConfig: InterfaceSymbol<ConnectConfig>;

/**
 * The connection status of the Genesis client.
 * @public
 */
export declare type ConnectionStatus = 'connected' | 'disconnected' | 'busy';

/**
 * @internal
 */
export declare class ConnectMock implements Connect {
    isConnected: boolean;
    isWorking: boolean;
    socket: SocketMock;
    get isConnectedSubject(): BehaviorSubject<boolean>;
    get isConnected$(): Observable<boolean>;
    send<T>(message: Message<any>, needsHandling?: boolean): Promise<any>;
    setValidSession(valid: boolean): void;
    httpMode(): boolean;
    nextMetadata: Metadata;
    nextJSONSchema: any;
    nextMessage: Message;
    requestParams: RequestParams;
    commitEvent(eventName: string, params?: CommitParams): Promise<Message>;
    connect(host: string): Promise<boolean>;
    disconnect(): void;
    getMetadata(resourceName: string): Promise<Metadata>;
    request(resourceName: string, params?: RequestParams): Promise<Message>;
    getAvailableResources(params?: RequestParams): Promise<Message>;
    snapshot(resourceName: string, params?: any): Promise<Message>;
    stream(resourceName: string, onMessage: Function, onError: Function, params?: any): SocketObservable<Message>;
    getMoreRows(sourceRef: string): Promise<Message<any>>;
    dataLogoff(sourceRef: string): Promise<Message<any>>;
    getJSONSchema(resourceName: string): Promise<any>;
}

/**
 * Enable intellisense for static core values, while allowing apps to overlay unknown values when needed?
 * Not sure how much value there is here until I speak with core server...
 */
/**
 * @internal
 */
export declare type CoreUserPermissions = 'somePermission' | 'someOtherPermission';

/**
 * @internal
 */
export declare type CoreUserProfiles = 'someProfile' | 'someOtherProfile';

/**
 * Represents user credentials.
 * @public
 */
declare type CredentialData_2 = PasswordCredentialData & {
    password?: string;
};
export { CredentialData_2 as CredentialData }

/**
 * Represents a system for managing user credentials and authentication tokens.
 * @public
 */
export declare interface CredentialManager {
    /**
     * Creates a set of basic authentication credentials using the specified input data.
     * @public
     * @param inputs - The input data to use when creating the credentials.
     * @returns A set of basic authentication credentials (username/password).
     */
    createBasicCredentials(inputs: BasicCredentialsInput): BasicAuthInfo;
    /**
     * Creates a set of refresh authentication credentials using the specified input data.
     * @public
     * @param inputs - The custom input data to use when creating the credentials, if any.
     * @returns A set of refresh authentication credentials.
     */
    createRefreshCredentials(inputs?: RefreshCredentialsInput): RefreshAuthInfo;
    /**
     * Creates a set of SSO authentication credentials using the specified input data.
     * @public
     * @param inputs - The custom input data to use when creating the credentials, if any.
     * @returns A set of SSO authentication credentials.
     */
    createSSOCredentials(inputs?: SSOCredentialsInput): SsoAuthInfo;
    /**
     * Validates the specified set of basic authentication credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateBasicCredentials(credentials: BasicAuthInfo): boolean;
    /**
     * Validates the specified set of refresh authentication credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateRefreshCredentials(credentials: RefreshAuthInfo): boolean;
    /**
     * Validates the specified set of SSO authentication credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateSSOCredentials(credentials: SsoAuthInfo): boolean;
    /**
     * Validate user credential data returned from the container or a cookie.
     *
     * @remarks
     * This can be called to ensure the credential is in the format this version of the manager expects.
     *
     * @privateRemarks
     * This can occur if the credential was saved using custom App code or via an early version of the manager itself.
     * The credential may be missing certain field values.
     *
     * @public
     * @param data - The credential data to validate.
     * @param ensurePassword - Check for the existence of a password. PasswordCredentials will have this, but cookies are considered valid without.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateUserCredentialData(data: CredentialData_2, ensurePassword?: boolean): boolean;
    /**
     * Normalise legacy credential data.
     *
     * @remarks
     * Older applications may have legacy credential data that needs migrating to the latest format the manager expects.
     * If the input CredentialData is already formatted as such, it will be returned as is. Otherwise, we check if any
     * cookie input is already formatted as such, and if so return it as CredentialData. If neither inputs are valid,
     * we will attempt a best effort normalisation to create valid CredentialData, or abort and return undefined.
     *
     * @public
     * @param data - The credential data to normalise.
     * @param cookie - The unknown cookie data to normalise.
     * @returns A tuple of the best effort normalisation of the inputs to the CredentialData format, and a boolean to indicate if we can remove the cookie.
     */
    normaliseLegacyCredentialData(data?: CredentialData_2, cookie?: CredentialData_2): [CredentialData_2 | undefined, boolean];
    /**
     * Stores the specified credential data in the browser's credential manager.
     * @public
     * @param data - The credential data to store.
     * @returns A Promise that resolves to the stored credential object, or `undefined` if storage fails.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CredentialsContainer
     */
    storeCredentialsInContainer(data: CredentialData_2): Promise<Credential | undefined>;
    /**
     * Retrieves credential data from the browser's credential manager.
     * @public
     * @param options - Options for filtering the retrieved credentials, if any.
     * @returns A Promise that resolves to the retrieved credential data, or `null` if no matching credentials were found.
     */
    getCredentialsFromContainer(options?: GetCredentialOptions): Promise<CredentialData_2 | null>;
    /**
     * Stores the specified credential data in a browser cookie.
     * @public
     * @param data - The credential data to store.
     * @returns `true` if storage was successful, or `false` otherwise.
     */
    storeCredentialsInCookie(data: CredentialData_2): boolean;
    /**
     * Retrieves credential data from a browser cookie.
     * @public
     * @returns The retrieved credential data, or `undefined` if no matching data was found.
     */
    getCredentialsFromCookie(): CredentialData_2 | undefined;
    /**
     * Stores user credentials securely.
     * @public
     * @param data - The credential data to be stored.
     * @returns A Promise that resolves to a boolean indicating if the credentials were stored successfully.
     */
    storeCredentials(data: CredentialData_2): Promise<boolean>;
    /**
     * Retrieves stored credentials.
     * @public
     * @param options - An object containing options for retrieving credentials.
     * @returns A Promise that resolves to a CredentialData object if the credentials are found, or undefined otherwise.
     */
    getCredentials(options?: GetCredentialOptions): Promise<CredentialData_2 | undefined>;
    /**
     * Retrieves the current SSO token, if any.
     * @public
     * @returns A string containing the SSO token, or null if there is no SSO token.
     */
    getSSOToken(): string | null;
    /**
     * Retrieves the current username, if any.
     * @public
     * @returns A string containing the username, or null if there is no username.
     */
    getUsername(): string | null;
    /**
     * Retrieves the current refresh token, if any.
     * @public
     * @returns A string containing the refresh token, or null if there is no refresh token.
     */
    getRefreshToken(): string | null;
    /**
     * Retrieves the current MFA token, if any.
     * @public
     * @returns A string containing the MFA token, or null if there is no mfa token.
     */
    getMFAToken(): string | null;
    /**
     * Removes the user's cookie.
     * @public
     */
    removeCookie(): void;
    /**
     * Removes the current SSO token, if any.
     * @public
     */
    removeSSOToken(): void;
    /**
     * Removes the current username, if any.
     * @public
     */
    removeUsername(): void;
    /**
     * Removes the current refresh token, if any.
     * @public
     */
    removeRefreshToken(): void;
    /**
     * Removes the current MFA token, if any.
     * @public
     */
    removeMFAToken(): void;
    /**
     * Removes all user crendentials data.
     * @public
     */
    removeAll(): void;
    /**
     * Prevents silent access by requiring the user to authenticate before making API requests.
     * @public
     */
    preventSilentAccess(): Promise<void>;
}

/**
 * The DI token for the CredentialManager interface.
 * @internal
 */
export declare const CredentialManager: InterfaceSymbol<CredentialManager>;

/**
 * The configuration for the CredentialManager
 * @public
 */
export declare interface CredentialManagerConfig {
    cookie: CredentialManagerCookieConfig;
}

/**
 * The DI key which can be used for explicit configuration by the host application
 * @internal
 */
export declare const CredentialManagerConfig: InterfaceSymbol<CredentialManagerConfig>;

/**
 * The configuration options for the CredentialManager cookie
 * @public
 */
export declare interface CredentialManagerCookieConfig {
    path: string;
    name: string;
    expires: number | Date;
    storePasswords?: boolean;
}

/**
 * The default separator used for splitting credential parts.
 * @public
 */
export declare const credentialSeparator = ":";

/**
 * Criteria filters to fields.
 *
 * @param filters - Concatenated criteria filters.
 *
 * @public
 * @remarks
 * For filtering REQUEST_SERVER resources. Maps concatenated criteria filters
 * from a grid to a request fields object. Date/datetime groovy expressions will
 * append `_FROM` and `_TO` onto field names appropriately
 *
 * eg: converts
 *
 * ```
 * foo == 'bar' &&
 * asd == 'zxc' &&
 * Expr.containsIgnoreCase(FIELD, 'abc') &&
 * Expr.dateIsToday(DATE) &&
 * Expr.dateTimeIsGreaterEqual(DATETIME, '20231110-03:23') &&
 * Expr.dateTimeIsLessEqual(DATETIME, '20231115-03:23')
 * ```
 *
 * into
 *
 * ```
 * {
 *   foo: 'bar',
 *   asd: 'zxc',
 *   FIELD: '*abc*',
 *   DATE_FROM: '20231110-00:00',
 *   DATE_TO: '20231111-00:00',
 *   DATETIME_FROM: '20231110-03:23',
 *   DATETIME_TO: '20231115-03:23',
 * }
 * ```
 */
export declare function criteriaFiltersToFields(filters: string): Record<string, string>;

/**
 * Represents the current user (i.e. the user that is currently logged in).
 * @public
 */
export declare class CurrentUser implements User {
    permissions: UserPermission[];
    profiles: UserProfile[];
    username: string;
    constructor(options: UserOptions);
    hasAdminProfile(): boolean;
    hasPermission(permission: string | string[]): boolean;
    hasProfile(profile: string | string[]): boolean;
    /**
     * Checks if the given key or keys are present in the given array of keys.
     * @internal
     * @param keys - The array of keys to check.
     * @param key - The key or keys to check for.
     * @returns True if the key or keys are present in the array of keys.
     */
    private hasKey;
}

/**
 * Data server result filter.
 *
 * @param result - The raw data server result.
 * @param rowId - Row id. Defaults to ROW_REF.
 *
 * @public
 * @remarks
 * Centralised filter logic for row operations.
 */
export declare function dataServerResultFilter(result: RawDataServerResult, rowId?: string): FilteredDataServerResult;

/**
 * Datasource interface.
 * @public
 * @remarks
 * The public interface available on the injected store fragment.
 */
export declare interface Datasource {
    /**
     * Indicates whether the datasource has been initialized.
     * @public
     */
    initialized: boolean;
    /**
     * Metadata about the fields of the datasource.
     * @public
     */
    fieldMetadata: FieldMetadata[];
    /**
     * Indicates whether the datasource is read-only.
     * TODO: Should have is prefix
     * @public
     */
    readOnly: boolean;
    /**
     * The type of resource the datasource represents.
     * @public
     */
    resourceType: ResourceType;
    /**
     * A stream of data from the datasource.
     * @public
     */
    stream: SocketObservable<FilteredDataServerResult | RequestServerResult>;
    /**
     * Indicates whether the resourceName is valid or not.
     * @public
     */
    validResourceName(resourceName: string): boolean;
    /**
     * Initializes the datasource with the given options and fetches metadata if specified.
     * @public
     * @param options - The options to initialize the datasource with.
     * @param fetchMeta - Whether to fetch metadata for the datasource.
     * @returns A promise that resolves with a boolean indicating whether the initialization was successful.
     */
    init(options: DatasourceOptions, fetchMeta: boolean): Promise<boolean>;
    /**
     * Deinitializes the datasource.
     * @remarks This should be called to clean up any resources when the datasource is no longer needed.
     * @public
     */
    deinit(): any;
    /**
     * Fetches metadata for the specified resource name.
     * @public
     * @param resourceName - The name of the resource to fetch metadata for.
     * @returns A promise that resolves with an array of FieldMetadata.
     */
    getMetadata?(resourceName: string): Promise<Array<FieldMetadata>>;
    /**
     * Takes a snapshot of the data in the datasource.
     * @public
     * @param params - Optional parameters for taking the snapshot.
     */
    snapshot(params?: any): any;
    /**
     * Takes a filtered snapshot of the data in the datasource.
     * @public
     * @param rowId - Optional ID of the row to filter.
     */
    snapshotFiltered(rowId?: string): any;
    /**
     * Updates the data in the datasource with the specified data.
     * @public
     * @param data - The data to update the datasource with.
     * @returns A promise that resolves when the data has been updated.
     */
    updateData?(data: Array<any>): Promise<void>;
    /**
     * Handles the specified data and returns it.
     * @public
     * @param data - The data to handle.
     * @returns The handled data.
     */
    dataHandler?(data: Array<any>): Array<any>;
    /**
     * Adds the specified dataview to the datasource.
     * @public
     * @param view - The dataview to add.
     */
    addView?(view: Dataview): void;
    /**
     * Removes the specified dataview from the datasource.
     * @public
     * @param view - The dataview to remove.
     */
    dropView?(view: Dataview): void;
    /**
     * Sets a filter on the datasource with the specified name, value, and type.
     * @public
     * @param name - The name of the filter to set.
     * @param value - The value of the filter to set.
     * @param type - The type of the filter to set.
     * @returns A promise that resolves when the filter has been set.
     */
    setFilter?(name: string, value: any, type?: string): Promise<void>;
    /**
     * Clears the filter with the specified name from the datasource.
     * @public
     * @param name - The name of the filter to clear.
     * @returns A promise that resolves when the filter has been cleared.
     */
    clearFilter?(name: string): Promise<void>;
    /**
     * Creates a new item in the datasource with the specified data.
     * @public
     * @param data - The data to create the item with.
     * @returns A promise that resolves with the ID of the created item.
     */
    createItem?(data: any): Promise<string>;
    /**
     * Deletes an item with the given ID from the datasource.
     * @public
     * @param itemId - The ID of the item to be deleted.
     * @returns A promise that resolves when the item has been deleted.
     */
    deleteItem?(itemId: string): Promise<void>;
    /**
     * Updates an item in the datasource with the given ID and data.
     * @public
     * @param itemId - The ID of the item to be updated.
     * @param data - The data to be updated for the item.
     * @returns A promise that resolves when the item has been updated.
     */
    updateItem?(itemId: string, data: any): Promise<void>;
}

/**
 * The DI token for the Datasource interface.
 * @internal
 */
export declare const Datasource: InterfaceSymbol<Datasource>;

/**
 * The configuration object for a Datasource
 * @public
 */
export declare interface DatasourceConfig {
    options: DatasourceOptions;
    criteriaToFields(filters: string): Record<string, string>;
    dataServerResultFilter(result: RawDataServerResult, rowId?: string): FilteredDataServerResult;
}

/**
 * The DI token for the DatasourceConfig interface.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const DatasourceConfig: InterfaceSymbol<DatasourceConfig>;

/**
 * Default values for a DatasourceConfig
 * @public
 */
export declare class DatasourceDefaults {
    static readonly MAX_ROWS_250 = 250;
    static readonly MAX_VIEW_1000 = 1000;
    static readonly REQ_REP_POLLING_INTERVAL_MS = 10000;
    static readonly DATASERVER_ROW_ID = "ROW_REF";
    static readonly REQUEST_SERVER_ROW_ID = "RECORD_ID";
}

/**
 * @internal
 */
export declare class DatasourceMock implements Datasource {
    initialized: boolean;
    fieldMetadata: FieldMetadata[];
    readOnly: boolean;
    resourceType: ResourceType;
    stream: SocketObservable<FilteredDataServerResult | RequestServerResult>;
    init(options: DatasourceOptions, fetchMeta: boolean): Promise<boolean>;
    deinit(): void;
    validResourceName(resourceName: string): boolean;
    getMetadata?(resourceName: string): Promise<FieldMetadata[]>;
    snapshot(params?: any): void;
    snapshotFiltered(rowId?: string): void;
    updateData?(data: any[]): Promise<void>;
    dataHandler?(data: any[]): any[];
    addView?(view: Dataview): void;
    dropView?(view: Dataview): void;
    setFilter?(name: string, value: any, type?: string): Promise<void>;
    clearFilter?(name: string): Promise<void>;
    createItem?(data: any): Promise<string>;
    deleteItem?(itemId: string): Promise<void>;
    updateItem?(itemId: string, data: any): Promise<void>;
}

/**
 * Options that can be passed when initializing a datasource.
 * @public
 */
export declare interface DatasourceOptions {
    /**
     * The criteria used to filter data in the datasource.
     */
    criteria?: string;
    /**
     * The fields to include in the returned data.
     */
    fields?: string;
    /**
     * Whether the datasource is a snapshot or not.
     */
    isSnapshot?: boolean;
    /**
     * The maximum number of rows to return from the datasource.
     */
    maxRows?: number;
    /**
     * The maximum number of rows to track as part of a client "view"
     */
    maxView?: number;
    /**
     * Defines the behaviour of the client view when new rows are received in real time.
     */
    movingView?: boolean;
    /**
     * The polling interval to use when using request/reply.
     * @remarks in milliseconds
     */
    pollingInterval?: number;
    /**
     * Whether to disable polling when using request/reply
     */
    disablePolling?: boolean;
    /**
     * The field to use for sorting the data.
     */
    orderBy?: string;
    /**
     * The request to send to the server.
     */
    request?: any;
    /**
     * The name of the resource to use for the datasource.
     */
    resourceName?: string;
    /**
     * Whether to return the data in reverse order or not.
     */
    reverse?: boolean;
}

/**
 * Data Update definition with details of what data to add, drop, and/or modify.
 * @public
 */
export declare interface DataUpdate {
    add?: Array<any>;
    drop?: Array<any>;
    modify?: Array<any>;
}

/**
 * Dataview object
 * @public
 */
export declare interface Dataview {
    /**
     * Function to update the data in the dataview object with an object of optional add, drop, and modify arrays of data.
     * @param data - A DataUpdate object containing the data to add, drop, and/or modify.
     * @returns A promise that resolves when the update is complete.
     */
    update?: (data: DataUpdate) => Promise<void>;
    /**
     * An array of data objects representing the data in the dataview.
     */
    data: any[];
    /**
     * An array of objects representing the metadata of each column in the dataview.
     */
    columnsMetadata?: FieldMetadata[];
    /**
     * A boolean indicating whether the search functionality should be hidden in the UI.
     */
    hideSearch?: boolean;
}

/**
 * The default implementation for the Auth interface.
 * @public
 */
export declare class DefaultAuth implements Auth {
    private session;
    private connect;
    private messageBuilder;
    private credentialManager;
    isWorking: boolean;
    currentUser: User;
    loggedUserResult: LoginResult;
    private loggedUserResultChanged;
    private isLoggedInSubject;
    get isLoggedIn$(): Observable<boolean>;
    get isLoggedIn(): boolean;
    constructor(session: Session, connect: Connect, messageBuilder: MessageBuilder, credentialManager: CredentialManager);
    login(credentials: AuthInfo): Promise<LoginResult>;
    logout(): Promise<LogoutResult>;
    reAuthFromSession(): Promise<boolean>;
    /**
     * Ensures that the connection is active.
     * @internal
     */
    private ensureConnection;
    /**
     * Gets the logged user from a Message result.
     * @param result - The Message result from auth request.
     * @returns The logged user info or null if the result is invalid.
     */
    private getLoggedUserFromMessage;
    /**
     * Applies the authentication results to the Auth instance.
     * @internal
     */
    private applyAuthResults;
    /**
     * Removes the authentication results from the Auth instance.
     * @internal
     */
    private removeAuthResults;
}

/**
 * The default implementation for the Connect interface (WS-only).
 * @public
 */
export declare class DefaultConnect implements Connect {
    socket: Socket;
    private messageBuilder;
    private metaCache;
    private config;
    isWorking: boolean;
    get isConnected(): boolean;
    get isConnectedSubject(): BehaviorSubject<boolean>;
    get isConnected$(): Observable<boolean>;
    constructor(socket: Socket, messageBuilder: MessageBuilder, metaCache: MetaCache, config?: ConnectConfig);
    httpMode(): boolean;
    setValidSession(valid: boolean): void;
    send<T>(message: Message<any>, needsHandling?: boolean): Promise<any>;
    connect(host: string): Promise<boolean>;
    disconnect(): void;
    snapshot(resourceName: string, params?: any): Promise<Message>;
    stream(resourceName: string, onMessage: Function, onError: Function, params?: any): SocketObservable<Message>;
    getMoreRows(sourceRef: string): Promise<Message>;
    dataLogoff(streamSourceRef: string): Promise<Message>;
    getMetadata(resourceName: string, useCache?: boolean): Promise<Metadata>;
    getJSONSchema(resourceName: string, useCache?: boolean): Promise<SchemaResponse>;
    request(resourceName: string, params?: RequestParams): Promise<Message>;
    getAvailableResources(params?: any): Promise<Message>;
    commitEvent(eventName: string, params?: CommitParams): Promise<Message>;
}

/**
 * The default configuration for connecting to a WS service.
 * @public
 */
export declare const defaultConnectConfig: ConnectConfig;

/**
 * The default implementation for the CredentialManager interface.
 * @public
 */
export declare class DefaultCredentialManager implements CredentialManager {
    private session;
    private config;
    constructor(session: Session, config?: CredentialManagerConfig);
    createBasicCredentials(inputs: BasicCredentialsInput): BasicAuthInfo;
    createRefreshCredentials(inputs?: RefreshCredentialsInput): RefreshAuthInfo;
    createSSOCredentials(inputs?: SSOCredentialsInput): SsoAuthInfo;
    validateBasicCredentials(credentials: BasicAuthInfo): boolean;
    validateRefreshCredentials(credentials: RefreshAuthInfo): boolean;
    validateSSOCredentials(credentials: SsoAuthInfo): boolean;
    validateUserCredentialData(data: CredentialData_2, ensurePassword?: boolean): boolean;
    normaliseLegacyCredentialData(data?: CredentialData_2, cookie?: CredentialData_2): [CredentialData_2 | undefined, boolean];
    storeCredentialsInContainer(data: CredentialData_2): Promise<Credential | undefined>;
    getCredentialsFromContainer(options?: CredentialRequestOptions): Promise<CredentialData_2 | null>;
    storeCredentialsInCookie(data: CredentialData_2): boolean;
    getCredentialsFromCookie(): CredentialData_2 | undefined;
    storeCredentials(data: CredentialData_2): Promise<boolean>;
    getCredentials(options?: GetCredentialOptions): Promise<CredentialData_2 | undefined>;
    getSSOToken(): string;
    getUsername(): string;
    getRefreshToken(): string;
    getMFAToken(): string;
    removeCookie(): void;
    removeSSOToken(): void;
    removeUsername(): void;
    removeRefreshToken(): void;
    removeMFAToken(): void;
    removeAll(): void;
    preventSilentAccess(): Promise<void>;
}

/**
 * The default configuration for the CredentialManager service.
 * @public
 */
export declare const defaultCredentialManagerConfig: CredentialManagerConfig;

/**
 * The default request options for the CredentialManager service.
 * @public
 */
export declare const defaultCredentialRequestOptions: GetCredentialOptions;

/**
 * The default implementation for the Datasource interface.
 * @public
 */
export declare class DefaultDatasource implements Datasource {
    protected connect: Connect;
    protected auth: Auth;
    protected config: DatasourceConfig;
    initialized: boolean;
    readOnly: boolean;
    protected options: DatasourceOptions;
    resourceType: ResourceType;
    protected _stream: SocketObservable<FilteredDataServerResult | RequestServerResult>;
    protected originalFieldDef: MetadataDetail[];
    protected requestFields: MetadataDetail[];
    fieldMetadata: FieldMetadata[];
    protected availableIndexes: IndexDetail[];
    constructor(connect: Connect, auth: Auth, config?: DatasourceConfig);
    validResourceName(resourceName: string): boolean;
    init(options: DatasourceOptions, fetchMeta?: boolean): Promise<boolean>;
    deinit(): void;
    get stream(): SocketObservable<FilteredDataServerResult | RequestServerResult>;
    get params(): any;
    snapshot(): Promise<Message>;
    snapshotFiltered(rowId?: string): Promise<any[]>;
    private createStream;
    private createDataserverStream;
    private createReqRepStream;
}

/**
 * The default configuration for the Datasource service.
 * @public
 */
export declare const defaultDatasourceConfig: DatasourceConfig;

/**
 * Default EntityDatasource DI implementation.
 *
 * @alpha
 */
export declare class DefaultEntityDatasource<TDTO, TEntity> extends DefaultDatasource implements EntityDatasource<TDTO, TEntity> {
    protected mapper: DTOMapper<TDTO, TEntity>;
    protected sourceRef: string;
    protected subscription: SocketSubscription;
    protected subscriptionLoggedOff: boolean;
    protected entityCache: TEntity[];
    protected entityMap: Map<string, TEntity>;
    protected rowId: string;
    /** {@inheritDoc EntityDatasource.initialized} */
    initialized: boolean;
    /** {@inheritDoc EntityDatasource.initialize} */
    initialize: (init: EntityDatasourceInit<TDTO, TEntity>) => Promise<boolean>;
    /** {@inheritDoc EntityDatasource.cache} */
    get cache(): TEntity[];
    /** {@inheritDoc EntityDatasource.isEmpty} */
    get isEmpty(): boolean;
    /** {@inheritDoc EntityDatasource.disconnect} */
    disconnect: () => void;
    protected updateCache(result: FilteredDataServerResult): void;
    protected processRowDataUpdate(rowData: RowData[], inserting?: boolean): void;
}

/**
 * The default implementation for the FoundationAnalytics interface.
 * @public
 */
export declare class DefaultFoundationAnalytics implements FoundationAnalytics {
    private analytics;
    /**
     * TODO: Remove the static config.
     * We could register a config with the container at startup and have it injected here or remove the constructor and
     * define a method call.
     */
    constructor(config: {
        app: 'foundation-app';
        debug: true;
    });
    /**
     * Set typed overloads
     */
    trackEvent(eventName: FoundationAnalyticsEventType.controlClicked, payload: FoundationAnalyticsEvent.ControlClicked): void;
    trackEvent(eventName: FoundationAnalyticsEventType.routeChanged, payload: FoundationAnalyticsEvent.RouteChanged): void;
}

/**
 * The default implementation of the Http interface.
 * @public
 */
export declare class DefaultHttp implements Http {
    private serializer;
    constructor(serializer: JSONSerializer);
    get<T>(url: string, requestInit: HttpRequestInit): Promise<T>;
    post<T>(url: string, requestInit: HttpRequestInit): Promise<T>;
}

/**
 * The default implementation for the Connect interface (HTTP-only).
 * @public
 */
export declare class DefaultHttpConnect implements Connect {
    http: Http;
    private messageBuilder;
    private session;
    private metaCache;
    private serializer;
    private config;
    host: string;
    isWorking: boolean;
    private _isConnected;
    isConnectedSubject: BehaviorSubject<boolean>;
    isConnected$: Observable<boolean>;
    get isConnected(): boolean;
    set isConnected(value: boolean);
    private ongoingTurbo?;
    /**
     * A map of active messages in "polling" mode.
     * @remarks Equivalent to messaged being streamed in a WebSocket connection.
     * @internal
     */
    private streams;
    constructor(http: Http, messageBuilder: MessageBuilder, session: Session, metaCache: MetaCache, serializer: JSONSerializer, config?: HttpConnectConfig);
    /**
     * Checks if the host is available by sending a heartbeat ping message and waiting for the response.
     * @internal
     * @returns A Promise that resolves to `true` if the host is available, and `false` otherwise.
     */
    private checkHost;
    send<T>(message: Message<any>, needsHandling?: boolean): Promise<any>;
    httpMode(): boolean;
    setValidSession(valid: boolean): void;
    connect(host: string): Promise<boolean>;
    disconnect(): void;
    snapshot(resourceName: string, params?: any): Promise<Message>;
    stream(resourceName: string, onMessage: Function, onError: Function, params?: any): SocketObservable<Message>;
    private getMessage;
    getMoreRows(sourceRef: string): Promise<Message>;
    dataLogoff(streamSourceRef: string): Promise<Message>;
    getMetadata(resourceName: string, useCache?: boolean): Promise<Metadata>;
    getJSONSchema(resourceName: string): Promise<SchemaResponse>;
    request(resourceName: string, params?: RequestParams): Promise<Message>;
    getAvailableResources(params?: any): Promise<Message>;
    commitEvent(eventName: string, params?: CommitParams): Promise<Message>;
    private maybeTurbo;
    private updateIntervals;
    private handlePollingIntervalMapUpdate;
    private static getBody;
    private static getUrl;
}

/**
 * The default configuration for connecting to an HTTP service.
 * @public
 */
export declare const defaultHttpConnectConfig: HttpConnectConfig;

/**
 * The default implementation for the KVStorage interface.
 * @alpha
 */
export declare class DefaultKVStorage implements KVStorage {
    private connect;
    private auth;
    private kvStorageConfig;
    constructor(connect: Connect, auth: Auth, kvStorageConfig?: KVStorageConfig);
    /**
     * Determine whether KV Storage is available for use.
     * @internal
     */
    private checkAvailability;
    /**
     * Deletes the specified key-value pairs from the KV storage.
     * @alpha
     * @param kvs - The key names to delete.
     * @returns A promise that resolves to true if the operation was successful, or an object containing error and warning information if the operation failed.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.delete(['myKey']);
     *  logger.debug(result); // true or { ERROR: ..., WARNING: ... }
     *  ...
     * ```
     */
    delete(keys: string[]): Promise<true | KVEventFailureResponse>;
    /**
     * Gets the specified key-value pairs from the KV storage.
     * @remarks If no key is specified, all key-value pairs for the current user are returned.
     * @alpha
     * @param kv - The single key-value pair to get.
     * @returns A promise that resolves to an array of key-value pairs.
     *
     * @example
     * ```ts
     * ...
     *  const result = await this.kvStorage.get({ key: 'myKey' });
     *  logger.debug(result); // [{ key: 'myKey', value: 'myValue' }]
     *
     *  const result2 = await this.kvStorage.get();
     *  logger.debug(result); // [{ key: 'myKey', value: 'myValue' }, { key: 'myOtherKey', value: 'myOtherValue' }]
     *  ...
     * ```
     */
    get<T = KVValue>(key?: string): Promise<KeyValueTransfer<T>[]>;
    /**
     * Lists all keys for the user.
     * @alpha
     * @returns A promise that resolves to all keys for the user.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.list();
     *  logger.debug(result); // ['myKey', 'myOtherKey']
     *  ...
     * ```
     */
    list(): Promise<string[]>;
    /**
     * Inserts or updates the specified key-value pairs in the KV storage.
     * @alpha
     * @param kvs - The key-value pairs to insert or update.
     * @returns A promise that resolves to true if the operation was successful, or an object containing error and warning information if the operation failed.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.put([{ key: 'myKey', value: 'myValue' }]);
     *  logger.debug(result); // true or { ERROR: ..., WARNING: ... }
     *  ...
     * ```
     */
    put(kvs: KeyValue<KVValue>[]): Promise<true | KVEventFailureResponse>;
}

/**
 * The default configuration for the KVStorage service.
 * @alpha
 */
export declare const defaultKVStorageConfig: KVStorageConfig;

/**
 * The default implementation for the MetaCache interface.
 * @public
 */
export declare class DefaultMetaCache implements MetaCache {
    cachedMetadata: Map<string, Metadata>;
    cachedJsonSchema: Map<string, SchemaResponse>;
    getMetadataFor(resourceName: string): Metadata;
    getJsonSchemaFor(resourceName: string): SchemaResponse;
    hasMetadataFor(resourceName: string): boolean;
    hasJsonSchemaFor(resourceName: string): boolean;
    setMetadataFor(resourceName: string, metadata: Metadata): void;
    setJsonSchemaFor(resourceName: string, schemaResponse: SchemaResponse): void;
}

/**
 * The default implementation for the Session interface.
 * @public
 */
export declare class DefaultSession implements Session {
    /**
     * The prefix to use for all storage keys.
     * @internal
     */
    private storageKeyPrefix;
    returnUrl: string;
    deleteUserFromSessionStorage(): void;
    setStorageKeyPrefix(prefix: string): void;
    setAuthResult(result: Message): void;
    captureReturnUrl(): void;
    setItem(key: string | UserInfoKeys, value: string, storage: StorageType): void;
    getItem(key: string | UserInfoKeys, storage: StorageType): string;
    removeItem(key: string | UserInfoKeys, storage: StorageType): void;
    setLocalStorageItem(key: string | UserInfoKeys, value: string): void;
    setSessionStorageItem(key: string | UserInfoKeys, value: string): void;
    getLocalStorageItem(key: string | UserInfoKeys): string;
    getSessionStorageItem(key: string | UserInfoKeys): string;
    removeLocalStorageItem(key: string | UserInfoKeys): void;
    removeSessionStorageItem(key: string | UserInfoKeys): void;
}

/**
 * EntityDatasource DI interface.
 *
 * @remarks
 * A simple streaming datasource cache used mainly by services that employ DTO mappers.
 *
 * @example
 * An example service using an underlying EntityDatasource:
 * ```ts
 * export class DefaultDomainService implements DomainService {
 *   constructor(
 *     @EntityDatasource protected datasource: EntityDatasource<DomainDTO, DomainEntity>,
 *     @DomainDTOMapper protected mapper: DomainDTOMapper,
 *     @optional(DomainServiceConfig) private config = defaultConfig
 *   ) {}
 *
 *   get initialized = () {
 *     return this.datasource.initialized;
 *   }
 *
 *   initialize = async (init) => {
 *     await this.datasource.initialize({
 *       mapper: this.mapper,
 *       options: {
 *         resourceName: this.config.resourceName,
 *         ...init,
 *       },
 *     });
 *     return this.initialized;
 *   };
 *
 *   list = () => this.datasource.cache;
 *
 *   get = async (id: string) => this.datasource.cache.find((entity) => entity.id === id);
 *
 *   action = async (id: string, message?: string): Promise<Message> => {
 *     return this.commitEvent('EVENT_DOMAIN_ACTION', { id, message });
 *   };
 *
 *   private async commitEvent(event: string, entity: Partial<DomainEntity>) {
 *     const msg = await this.connect.commitEvent(event, {
 *       DETAILS: this.mapper.toDTO(entity),
 *       IGNORE_WARNINGS: true,
 *       VALIDATE: false,
 *     });
 *     return messageOrThrow(msg);
 *   }
 * ```
 *
 * @privateRemarks
 * This is just a first pass that extends DefaultDatasource, but it may evolve into something standalone. It doesn't
 * cover snapshots by design right now, and there's some overlap with existing datasource logic atm. I haven't covered
 * all scenarios like data log just yet, as I'd like us to re-think some logic before doing that.
 *
 * Router converter functions would likely leverage services to resolve entities ahead of the enter phase.
 * ```ts
 * { path: '/{entity:DomainEntity}', element: MyRoute }
 * ...
 * this.routes.converter('DomainEntity', async (entityId: string) => {
 *   const entity = await this.domainService.get(entityId);
 *   return entity;
 * });
 * ```
 *
 * @alpha
 */
export declare interface EntityDatasource<TDTO, TEntity> {
    /**
     * @remarks
     * Datasource initialized.
     */
    readonly initialized: boolean;
    /**
     * @remarks
     * Initialize datasource.
     */
    initialize(init: EntityDatasourceInit<TDTO, TEntity>): Promise<boolean>;
    /**
     * @remarks
     * In-memory entity cache.
     *
     * @privateRemarks
     * Keeping it simple for now, but this could be an indexedDB table, perhaps with Dexie + liveQuery in-front.
     */
    readonly cache: TEntity[];
    /**
     * @remarks
     * Cache is empty / awaiting first result.
     */
    readonly isEmpty: boolean;
    /**
     * @remarks
     * Disconnect and reset cache.
     */
    disconnect(): void;
}

/**
 * EntityDatasource DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const EntityDatasource: InterfaceSymbol<EntityDatasource<any, any>>;

/**
 * This is a wip. We need to see if and where this fits into our strategic shared datasource solution.
 */
/**
 * EntityDatasourceInit interface.
 *
 * @alpha
 */
export declare interface EntityDatasourceInit<TDTO, TEntity> {
    options: Omit<DatasourceOptions, 'isSnapshot'>;
    mapper: DTOMapper<TDTO, TEntity>;
    fetchMeta?: boolean;
}

/**
 * Event Message types
 * @public
 */
export declare enum EventMessageType {
    EVENT_LOGIN_AUTH = "EVENT_LOGIN_AUTH",
    EVENT_LOGIN_AUTH_ACK = "EVENT_LOGIN_AUTH_ACK",
    EVENT_LOGIN_AUTH_NACK = "EVENT_LOGIN_AUTH_NACK",
    EVENT_LOGOUT = "EVENT_LOGOUT",
    DATA_LOGON = "DATA_LOGON",
    DATA_LOGOFF = "DATA_LOGOFF",
    META_REQUEST = "META_REQUEST",
    JSON_SCHEMA_REQUEST = "JSON_SCHEMA_REQUEST",
    RESOURCES_REQUEST = "RESOURCES_REQUEST",
    EVENT_CHANGE_USER_PASSWORD = "EVENT_CHANGE_USER_PASSWORD",
    EVENT_SELF_SERVICE_PASSWORD_RESET = "EVENT_SELF_SERVICE_PASSWORD_RESET",
    EVENT_PASSWORD_RESET_ACTION = "EVENT_PASSWORD_RESET_ACTION"
}

/**
 * Expontential Interval scheduler for reconnect
 * @public
 * @param reconnectOptions - {@link SocketReconnectOptions} options
 * @param attempt - number of attempts
 * @returns number of milliseconds that is a power of 2 ^ attempts * baseInterval
 */
export declare const exponentialScheduler: SocketReconnectScheduler;

/**
 * Extracts the field definitions from the metadata.
 * @public
 * @param metadata - The metadata object.
 * @returns An array of metadata detail.
 */
export declare const extractFieldDefinitions: (metadata: Metadata) => MetadataDetail[];

/**
 * The DI token for the FDC3Service interface.
 * @internal
 */
export declare const FDC3: InterfaceSymbol<FDC3_2>;

/**
 * The FDC3 API interface.
 * @public
 */
declare interface FDC3_2 {
    /**
     * Whether the FDC3 API is ready.
     * @public
     */
    isReady: boolean;
    /**
     * Adds intent listeners for the specified intents.
     * @public
     * @param listeners - The intent listeners to add.
     */
    addIntentListeners(listeners: Map<FDC3Intents, FDC3ContextHandler>): void;
    /**
     * Finds an app that supports the specified intent.
     * @public
     * @param intent - The intent to find an app for.
     * @param context - The context to use.
     * @returns A promise that resolves with the app and its metadata.
     */
    findIntent(intent: FDC3Intents, context?: FDC3Context): Promise<FDC3AppIntent>;
    /**
     * Finds all the intents supported by the specified context.
     * @public
     * @param context - The context to use.
     * @returns A promise that resolves with an array of all matching intents and their metadata.
     */
    findIntentsByContext(context: FDC3Context): Promise<FDC3AppIntent[]>;
    /**
     * Raises the specified intent with the specified context.
     * @public
     * @param context - The context to use.
     * @param intent - The intent to raise.
     * @returns A promise that resolves with the result of the intent resolution.
     */
    raiseIntent(context: FDC3Context, intent?: FDC3Intents): Promise<FDC3IntentResolution>;
    /**
     * Joins the specified channel.
     * @public
     * @param channelId - The ID of the channel to join.
     * @returns A promise that resolves when the join is successful.
     */
    joinChannel(channelId: string): Promise<void>;
    /**
     * Gets or creates the specified channel.
     * @public
     * @param  channelId - The ID of the channel to get or create.
     * @returns A promise that resolves with the channel object.
     */
    getOrCreateChannel(channelId: string): Promise<FDC3Channel>;
    /**
     * Broadcasts the specified context on the specified channel.
     * @public
     * @param channelId - The ID of the channel to broadcast on.
     * @param context - The context to broadcast.
     * @returns A promise that resolves when the broadcast is successful.
     */
    broadcastOnChannel(channelId: string, context: FDC3Context): Promise<void>;
}

/**
 * The DI token for the FDC3Service interface.
 * @internal
 */
declare const FDC3: InterfaceSymbol<FDC3_2>;

/**
 * @public
 */
export declare interface FDC3AppIntent extends AppIntent {
}

/**
 * @public
 */
export declare interface FDC3Channel extends Channel {
}

/**
 * @public
 */
export declare interface FDC3Context extends Context {
}

/**
 * @public
 */
export declare type FDC3ContextHandler = ContextHandler;

/**
 * @public
 */
export declare interface FDC3IntentResolution extends IntentResolution {
}

/**
 * @public
 */
export declare type FDC3Intents = Intents;

/**
 * Field Metadata definition.
 * @public
 */
export declare type FieldMetadata = {
    name: string;
    displayName?: string;
    type: FieldTypeEnum;
    enumOptions?: Array<any>;
    enumMultiple?: boolean;
    required?: boolean;
    readOnly?: boolean;
    component?: string;
};

/**
 * Available types for a field metadata.
 * @public
 */
export declare enum FieldTypeEnum {
    STRING = "STRING",
    NUMBER = "NUMBER",
    BOOLEAN = "BOOLEAN",
    ENUM = "ENUM",
    DATE = "DATE",
    DATETIME = "DATETIME",
    CUSTOM = "CUSTOM"
}

/**
 * Fitlered result after initial DATASERVER data fetch.
 * @public
 */
export declare type FilteredDataServerResult = {
    inserts?: RowData[];
    deletes?: RowData[];
    updates?: RowData[];
    SOURCE_REF?: string;
};

/**
 * Represents an analytics tracking system.
 * @public
 */
export declare interface FoundationAnalytics {
    /**
     * Tracks a "control clicked" event with the specified payload.
     * @public
     * @param eventName - The name of the event to track.
     * @param payload - The payload data for the event.
     */
    trackEvent(eventName: FoundationAnalyticsEventType.controlClicked, payload: FoundationAnalyticsEvent.ControlClicked): void;
    /**
     * Tracks a "route changed" event with the specified payload.
     * @public
     * @param eventName - The name of the event to track.
     * @param payload - The payload data for the event.
     */
    trackEvent(eventName: FoundationAnalyticsEventType.routeChanged, payload: FoundationAnalyticsEvent.RouteChanged): void;
    /**
     * Tracks an event with the specified name and payload.
     * @public
     * @param eventName - The name of the event to track.
     * @param payload - The payload data for the event.
     */
    trackEvent(eventName: FoundationAnalyticsEventType, payload: {}): void;
}

/**
 * The DI token for the FouundationAnalytics interface.
 * @internal
 */
export declare const FoundationAnalytics: InterfaceSymbol<FoundationAnalytics>;

/** Namespace for all Analytics event types
 * @public
 */
export declare namespace FoundationAnalyticsEvent {
    /**
     * Represents an event that is triggered when a control is clicked.
     * @public
     */
    export interface ControlClicked {
        name: string;
    }
    /**
     * Represents an event that is triggered when a control is clicked.
     * @public
     */
    export interface RouteChanged {
        path: string;
        referrer?: string;
    }
}

/**
 * Analytics event types
 * @public
 */
export declare enum FoundationAnalyticsEventType {
    controlClicked = "controlClicked",
    routeChanged = "routeChanged"
}

/**
 * Get Connect from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getConnect } from '@genesislcap/foundation-comms';
 * import { API_HOST } from '@genesislcap/foundation-utils';
 * ...
 * private connect = getConnect();
 * ...
 * const connected = await this.connect.connect(API_HOST); // < WebSocket url
 * ```
 *
 * @public
 */
export declare function getConnect(): Connect;

/**
 * Represents options for retrieving credentials.
 * @public
 */
export declare type GetCredentialOptions = CredentialRequestOptions & {};

/**
 * Get Datasource from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { FilteredDataServerResult, getDatasource, SocketObservable } from '@genesislcap/foundation-comms';
 * ...
 * private ds = getDatasource();
 * private dsSub: SocketObservable<FilteredDataServerResult>;
 * ...
 * const dsInit = await this.ds.init({
 *   criteria: '',
 *   isSnapshot: false,
 *   resourceName: 'ALL_POSITIONS',
 *   reverse: false,
 *   maxRows: 250,
 *   maxView: 1000,
 * });
 * if (!dsInit) {
 *   logger.debug(`Failed to initialise 'ALL_POSITIONS' datasource`);
 *   return;
 * }
 * this.dsSub = this.ds.stream.subscribe((result) => { ... });
 * ```
 *
 * @public
 */
export declare function getDatasource(): Datasource;

/**
 * An interface representing an HTTP client that can make GET and POST requests and return a promise with the response data.
 * @public
 */
export declare interface Http {
    get<T = any>(url: string, requestInit: HttpRequestInit): Promise<T>;
    post<T = any>(url: string, requestInit: HttpRequestInit): Promise<T>;
}

/**
 * The DI token for the Http interface.
 * @internal
 */
export declare const Http: InterfaceSymbol<Http>;

/**
 * The DI token for the Connect interface (HTTP-only).
 * @internal
 */
export declare const HttpConnect: InterfaceSymbol<Connect>;

/**
 * Configuration options for the Connect (HTTP) instance.
 * @public
 */
export declare interface HttpConnectConfig {
    /**
     * Configuration options for http polling.
     */
    polling: HttpPollingConfig;
}

/**
 * Configuration options for the Connect (HTTP) instance.
 * @internal
 */
export declare const HttpConnectConfig: InterfaceSymbol<HttpConnectConfig>;

/**
 * Configuration options for http connection.
 * @public
 */
export declare interface HttpPollingConfig {
    /**
     * The frequency of polling.
     */
    POLLING_FREQUENCY?: number;
    /**
     * The interval map of polling.
     */
    POLLING_INTERVAL_MAP?: {
        [resource: string]: number;
    };
    /**
     * The map of polling configuration.
     */
    POLLING_MAP?: {
        [key: string]: PollingConfiguration;
    };
}

/**
 * The options object used when making an HTTP request.
 * @public
 */
export declare interface HttpRequestInit extends RequestInit {
    /**
     * A serializer function to transform the request body.
     * @remarks
     * This can be used to transform the request body to a specific format such as JSON or XML.
     */
    serializer?: JSONSerializer;
}

/**
 * Index Detail definition.
 * @public
 */
export declare type IndexDetail = {
    NAME: string;
    FIELDS: string;
};

/**
 * Regular Expression to test valid WebSocket (wss and ws) hosts
 * @public
 */
export declare const isWebSocketExpression: RegExp;

/**
 * A JSON schema cache definition
 * @public
 */
export declare interface JsonSchemaCache {
    cachedJsonSchema: Map<string, SchemaResponse>;
    getJsonSchemaFor(resourceName: string): SchemaResponse;
    hasJsonSchemaFor(resourceName: string): boolean;
    setJsonSchemaFor(resourceName: string, schemaResponse: SchemaResponse): any;
}

/**
 * Represents JWT authentication information.
 * @public
 */
export declare type JwtAuthInfo = {
    type: AuthType & AuthType.JWT;
    jwt: string;
    domain: string;
};

/**
 * KeyValue interface.
 * @alpha
 */
export declare interface KeyValue<T = KVValue> {
    /**
     * Key
     * @alpha
     */
    key: string;
    /**
     * Value
     * @alpha
     */
    value?: T;
    /**
     * Time key expires.
     *
     * @privateRemarks
     * Should we clean up with lazy deletion, ie. wait until an access request?
     *
     * @alpha
     */
    expiration?: number;
    /**
     * Value should be encoded.
     * @alpha
     */
    base64?: boolean;
    /**
     * Additional metadata.
     * @alpha
     */
    metadata?: {
        [key: string]: unknown;
    };
}

/**
 * KeyValueTransfer interface.
 *
 * @alpha
 */
export declare interface KeyValueTransfer<T> {
    kv: KeyValue<T>;
    serialize?(value: unknown): unknown;
    deserialize?(value: unknown): Promise<T>;
}

/**
 * KVStorage type.
 * @alpha
 */
export declare type KVEventFailureResponse = {
    ERROR?: MessageError[];
    WARNING: any[];
};

/**
 * KV Event Message types
 * @alpha
 */
export declare enum KVEventMessageType {
    EVENT_KEY_VALUE_PUT = "EVENT_KEY_VALUE_PUT",
    EVENT_KEY_VALUE_DELETE = "EVENT_KEY_VALUE_DELETE"
}

/**
 * KV Message types
 * @alpha
 */
export declare enum KVMessageType {
    REQ_KEY_VALUE_LIST = "REQ_KEY_VALUE_LIST",
    REQ_KEY_VALUE_READ = "REQ_KEY_VALUE_READ"
}

/**
 * Represents a KV (key-value) data storage and its capabilities.
 * Provides methods to set and get items from the KV data storage.
 *
 * @remarks
 * Provides a key value storage api that abstracts the underlying storage technology.
 *
 * @privateRemarks
 * This entire package will likely evolve into a standalone repo; Storage PBC.
 * Need to create a design document covering how account and namespace ids may come into play.
 *
 * @alpha
 */
export declare interface KVStorage {
    delete(keys: string[]): Promise<true | KVEventFailureResponse>;
    get<T = KVValue>(key?: string): Promise<KeyValueTransfer<T>[]>;
    list(): Promise<string[]>;
    put(kvs: KeyValue<KVValue>[]): Promise<true | KVEventFailureResponse>;
}

/**
 * The DI token for the KVStorage interface.
 * @internal
 */
export declare const KVStorage: InterfaceSymbol<KVStorage>;

/**
 * KVStorageConfig DI interface.
 * @alpha
 */
export declare interface KVStorageConfig {
    /**
     * The account id to use.
     *
     * @privateRemarks
     * May require access control.
     *
     * @alpha
     */
    account?: string;
    /**
     * The namespace to use.
     *
     * @privateRemarks
     * May require access control.
     *
     * @alpha
     */
    namespace?: string;
    /**
     * Access control auth key.
     *
     * @alpha
     */
    authKey?: string;
    /**
     * Access control auth email.
     *
     * @remarks
     * Part of access requirements and where to report integration issues.
     *
     * @alpha
     */
    authEmail?: string;
}

/**
 * KVStorageConfig DI key.
 *
 * @internal
 */
export declare const KVStorageConfig: InterfaceSymbol<KVStorageConfig>;

/**
 * KVValue type.
 * @alpha
 */
export declare type KVValue = string | ArrayBuffer;

/**
 * Linear Interval scheduler for reconnect
 * @public
 * @param reconnectOptions - {@link SocketReconnectOptions} options
 * @returns  number of milliseconds specified in reconnectOptions up to @see MAX_RECONNECT_INTERVAL
 */
export declare const linearScheduler: SocketReconnectScheduler;

/**
 * Logger for the foundation-comms package
 * @public
 */
export declare const logger: Logger;

/**
 * Represents the result of a user login operation.
 * @public
 */
export declare type LoginResult = {
    authToken: string;
    refreshToken: string;
    username: string;
    profiles: string[];
    permissions: string[];
    sessionId: string;
};

/**
 * Represents the result of a user logout operation.
 * @public
 */
export declare interface LogoutResult {
    /**
     * Indicates if the logout operation was successful.
     * @public
     */
    success: boolean;
    /**
     * Provides more details if the logout operation failed.
     * @public
     */
    error?: string;
}

/**
 * Default number of reconnect attempts
 * @public
 */
export declare const MAX_RECONNECT_ATTEMPTS = 10;

/**
 * Base Message type for all DETAIL variants
 * @public
 */
export declare type Message<T = void | any> = {
    REPLY?: any[];
    CODE?: MessageCode | string;
    DETAILS?: T;
    LOGGED_OUT?: boolean;
    ERROR?: MessageError[];
    MESSAGE_TYPE?: MessageType | EventMessageType | string;
    PERMISSION?: string[];
    PROFILE?: string[];
    RESOURCES?: any[];
    REFRESH_AUTH_TOKEN?: string;
    ROW?: any[];
    ROWS_COUNT?: number;
    SEQUENCE_ID?: number;
    SERVICE_NAME?: string;
    SESSION_AUTH_TOKEN?: string;
    SESSION_ID?: string;
    SOURCE_REF?: string;
    USER_NAME?: string;
    WARNING?: any[];
};

/**
 * A utility class for building different types of messages.
 * @public
 */
export declare interface MessageBuilder {
    /**
     * Creates a login message.
     *
     * @remarks EVENT_LOGIN_AUTH
     * @public
     * @param credentials - The authentication credentials.
     * @returns The login message.
     */
    createLoginMessage(credentials: AuthInfo): Message<MessageDetails.Login>;
    /**
     * Creates a logout message.
     *
     * @remarks EVENT_LOGOUT
     * @public
     * @param loginResult - The login result.
     * @returns The logout message.
     */
    createLogoutMessage(loginResult?: LoginResult): Message;
    /**
     * Creates a message to logon to a DATASERVER.
     * Used for data fetching.
     *
     * @remarks DATA_LOGON
     * @public
     * @param resourceName - The resource name.
     * @param params - The additional parameters.
     * @returns The data logon message.
     */
    createDataLogonMessage(resourceName: string, params?: any): Message<MessageDetails.DataserverRequest>;
    /**
     * Creates a message to logoff from a DATASERVER.
     *
     * @remarks DATA_LOGOFF
     * @public
     * @param sourceRef - The source reference.
     * @returns The data logoff message.
     */
    createDataLogoffMessage<T = any>(sourceRef: string): Message<T>;
    /**
     * Creates a message to request more rows.
     *
     * @remarks MORE_ROWS
     * @public
     * @param sourceRef - The source reference.
     * @returns The more rows message.
     */
    createMoreRowsMessage(sourceRef: string): Message;
    /**
     * Creates a request message.
     *
     * @remarks
     * For REQUEST_SERVER resources. You can send with REQ_ or without it.
     * @public
     * @param resourceName - The resource name.
     * @param params - The additional parameters.
     * @returns The request message.
     */
    createRequestMessage<T = any>(resourceName: string, params?: any): Message<T>;
    /**
     * Creates a message for getting list of resources.
     *
     * @public
     * @param params - The additional parameters.
     * @returns The request message.
     */
    createResourcesMessage<T = any>(params?: any): Message<T>;
    /**
     * Creates a metadata fetch message.
     *
     * @remarks META_REQUEST
     * @public
     * @param resourceName - The resource name.
     * @param messageType - The event message type.
     * @returns The meta request message.
     */
    createMetaRequestMessage(resourceName: string, messageType?: EventMessageType): Message<MessageDetails.MetaRequest>;
    /**
     * Creates an event commit message.
     *
     * @public
     * @param eventName - The event name.
     * @param params - The additional parameters.
     * @returns The commit message.
     */
    createCommitMessage<T = any>(eventName: string, params?: CommitParams): Message<T>;
    /**
     * Creates an auth message specific for token refresh.
     *
     * @remarks EVENT_LOGIN_AUTH
     * @public
     * @returns The refresh token message.
     */
    createRefreshTokenMessage(): Message<MessageDetails.RefreshToken>;
    /**
     * Creates a change password message.
     *
     * @remarks EVENT_CHANGE_USER_PASSWORD
     * @public
     * @param username - The username.
     * @param oldPassword - The old password.
     * @param newPassword - The new password.
     * @returns The change password message.
     */
    createChangePasswordMessage(username: string, oldPassword: string, newPassword: string): Message<MessageDetails.ChangePassword>;
    /**
     * Creates a forgot password message.
     *
     * @remarks EVENT_SELF_SERVICE_PASSWORD_RESET
     * @public
     * @param username - The username.
     * @param returnUrl - The return URL.
     * @param requester - The requester.
     * @returns The forgot password message.
     */
    createForgotPasswordMessage(username: string, returnUrl: string, requester?: string): Message<MessageDetails.ForgotPassword>;
    /**
     * Creates a forgot password token message.
     *
     * @public
     * @param username - The username.
     * @param resetToken - The reset token.
     * @param newPassword - The new password.
     * @param requester - The requester.
     * @returns The forgot password token message.
     */
    createForgotPasswordTokenMessage(username: string, resetToken: string, newPassword: string, requester?: string): Message<MessageDetails.ForgotPasswordToken>;
    /**
     * Create HTTP Headers from a message.
     *
     * @public
     * @param message - The message.
     * @param contentType - Optional content-type, defaults to `application/json; charset=UTF-8`.
     * @returns The headers.
     */
    createHTTPHeadersFromMessage(message: Message, contentType?: string): HeadersInit;
}

/**
 * The DI token for the MessageBUilder interface.
 * @internal
 */
export declare const MessageBuilder: InterfaceSymbol<MessageBuilder>;

/**
 * General Message codes
 * @public
 */
export declare enum MessageCode {
    SESSION_NOT_FOUND = "SESSION_NOT_FOUND",
    UNKNOWN_MESSAGE_TYPE = "UNKNOWN_MESSAGE_TYPE"
}

/** Namespace for all Message detail types
 * @public
 */
export declare namespace MessageDetails {
    /**
     * Details type for a basic auth message
     * @public
     */
    export type LoginBasic = {
        USER_NAME: string;
        PASSWORD: string;
    };
    /**
     * Details type for a SSO-based auth message
     * @public
     */
    export type LoginSSO = {
        SSO_TOKEN: string;
    };
    /**
     * Details type for a JWT-based auth message
     * @public
     */
    export type LoginJWT = {
        JWT: string;
        DOMAIN: string;
    };
    /**
     * Union type for different types of login information
     * @public
     */
    export type Login = LoginBasic | LoginSSO | LoginJWT;
    /**
     * Details type for a password-change message
     * @public
     */
    export type ChangePassword = {
        OLD_PASSWORD: string;
        NEW_PASSWORD: string;
    };
    /**
     * Details type for a password-reset message
     * @public
     */
    export type ResetPassword = {
        USER_NAME: string;
    };
    /**
     * Details type for a forgot-password message
     * @public
     */
    export type ForgotPassword = {
        USER_NAME: string;
        RETURN_URL: string;
    };
    /**
     * Details type for a forgot-password token-based message
     * @public
     */
    export type ForgotPasswordToken = {
        USER_NAME: string;
        RESET_TOKEN: string;
        NEW_PASSWORD: string;
    };
    /**
     * Details type for a password-expiry message
     * @public
     */
    export type ExpirePassword = {
        USER_NAME: string;
        PASSWORD: string;
    };
    /**
     * Details type for a refresh-token message
     * @public
     */
    export type RefreshToken = {
        REFRESH_AUTH_TOKEN: string;
        MFA_CODE?: string;
    };
    /**
     * Details type to request more rows
     * @public
     */
    export type MoreRows = {
        SOURCE_REF: string;
    };
    /**
     * Details type to update columns
     * @public
     */
    export type UpdateColumns = {
        SOURCE_REF: string;
        ADD_COLUMNS: string;
        DROP_COLUMNS: string;
    };
    /**
     * Details type to request DATASERVER metadata
     * @public
     */
    export type DataserverRequest = {
        DATASOURCE_NAME: string;
    };
    /**
     * Details type to request FEATURE metadata
     * @public
     */
    export type MetaRequest = {
        FEATURE: string;
    };
    /**
     * Details type for ERROR results
     * @public
     */
    export type Error = {
        ERROR: string;
    };
    /**
     * System-level information when a user is logged in.
     * @public
     */
    export type LoggedInSystemInfo = {
        DAYS_TO_PASSWORD_EXPIRY: number;
        FAILED_LOGIN_ATTEMPTS: number;
        HEARTBEAT_INTERVAL_SECS: number;
        LAST_LOGIN_DATETIME: number;
        NOTIFY_EXPIRY?: boolean;
        PRODUCT?: any[];
        SYSTEM?: any;
    };
}

/**
 * Error definition for a message result
 * @public
 */
export declare type MessageError = {
    CODE: MessageCode | string;
    STATUS_CODE?: string;
    TEXT: string;
};

/**
 * Check if the provided message contains an error and throw an Error if it does
 * @public
 * @param message - Message to check for errors
 * @returns The message if no errors are present, otherwise throws an Error
 */
export declare const messageOrThrow: (message: Message) => Message<any>;

/**
 * General Message types
 * @public
 */
export declare enum MessageType {
    LOGON_ACK = "LOGON_ACK",
    LOGON_NACK = "LOGON_NACK",
    LOGOFF_ACK = "LOGOFF_ACK",
    LOGOFF_NACK = "LOGOFF_NACK",
    HEARTBEAT_PING = "HEARTBEAT_PING",
    HEARTBEAT_PONG = "HEARTBEAT_PONG",
    MORE_ROWS = "MORE_ROWS",
    MSG_ACK = "MSG_ACK",
    MSG_NACK = "MSG_NACK",
    LOGOUT_ACK = "LOGOUT_ACK",
    LOGOUT_NACK = "LOGOUT_NACK",
    QUERY_UPDATE = "QUERY_UPDATE"
}

/**
 * Union type of MetadataCache and JsonSchemaCache
 * @public
 */
export declare type MetaCache = MetadataCache & JsonSchemaCache;

/**
 * The DI token for the MetaCache interface.
 * @internal
 */
export declare const MetaCache: InterfaceSymbol<MetaCache>;

/**
 * Metadata definition for a given resource.
 * @public
 */
export declare type Metadata = {
    FIELD?: MetadataDetail[];
    INDEXES?: IndexDetail[];
    REPLY_FIELD?: MetadataDetail[];
    REQUEST_FIELD?: MetadataDetail[];
    NAME?: string;
    TYPE?: string;
    ERROR?: string;
};

/**
 * A Metadata cache definition
 * @public
 */
export declare interface MetadataCache {
    /**
     * A map of cached metadata, with resource names as keys and metadata as values.
     * @public
     */
    cachedMetadata: Map<string, Metadata>;
    /**
     * Returns the metadata for the specified resource name.
     * @public
     * @param resourceName - The name of the resource.
     * @returns The metadata for the specified resource name.
     */
    getMetadataFor(resourceName: string): Metadata;
    /**
     * Returns true if metadata exists for the specified resource name, otherwise false.
     * @public
     * @param resourceName - The name of the resource.
     * @returns True if metadata exists for the specified resource name, otherwise false.
     */
    hasMetadataFor(resourceName: string): boolean;
    /**
     * Sets the metadata for the specified resource name.
     * @public
     * @param resourceName - The name of the resource.
     * @param metadata - The metadata to set.
     */
    setMetadataFor(resourceName: string, metadata: Metadata): void;
}

/**
 * Metadata Detail definition.
 * @public
 */
export declare type MetadataDetail = {
    NAME: string;
    TYPE: string;
    VALID_VALUES?: string;
    READ_ONLY?: boolean;
    OPTIONAL?: boolean;
    UI_LABEL?: string;
};

/**
 * Normalise criteria.
 *
 * @param criteria - Original concat criteria.
 * @param delimiter - Criteria delimiter.
 * @returns normalized concat criteria.
 *
 * @public
 * @remarks
 * Normalises criteria, as the order of filters should not matter, also gets rid of spaces.
 * @privateRemarks
 * Can we use the criteria builder for this and related criteria stuff within datasources?
 */
export declare function normaliseCriteria(criteria: string, delimiter?: string): string;

/**
 * Configuration options for http polling.
 * @public
 */
export declare interface PollingConfiguration {
    /**
     * The temporary polling frequency.
     */
    TEMP_FREQUENCY: number;
    /**
     * The expected amount of HTTP polling attempts to be performed.
     * @remarks TEMP_FREQUENCY * AMOUNT_OF_POLLS = the total polling time/duration
     */
    AMOUNT_OF_POLLS: number;
    /**
     * The queries to be polled.
     */
    QUERIES: string[];
}

/**
 * Raw result from a DATASERVER data fetch.
 * @public
 */
export declare type RawDataServerResult = {
    MESSAGE_TYPE: string;
    ROW: ServerRow[];
    SOURCE_REF: string;
    MORE_ROWS: boolean;
    ROWS_COUNT: number;
    SEQUENCE_ID: number;
};

/**
 * Represents refresh token authentication information.
 * @public
 */
export declare type RefreshAuthInfo = {
    type: AuthType & AuthType.REFRESH;
    username: string;
    refreshToken: string;
    mfaToken: string;
};

/**
 * Represents refresh credentials for a user.
 * @public
 */
export declare type RefreshCredentialsInput = {
    username: string;
    refreshToken: string;
    mfaToken?: string;
};

/**
 * Parameters for a REQUEST message
 * @public
 */
export declare type RequestParams = {
    DETAILS?: any;
    REQUEST?: any;
};

/**
 * Raw result from a REQUEST_SERVER data fetch.
 * @public
 */
export declare type RequestServerResult = {
    SOURCE_REF?: string;
    REPLY: any[];
    MESSAGE_TYPE?: string;
};

/**
 * Types of Genesis resources.
 * @public
 */
export declare enum ResourceType {
    DATASERVER = "DATASERVER",
    EVENT_HANDLER = "EVENT_HANDLER",
    REQUEST_SERVER = "REQUEST_SERVER"
}

/**
 * Generate reconnect interval based on scheduler
 * @public
 * @param reconnectOptions - {@link SocketReconnectOptions} options
 * @param scheduler - {@link SocketReconnectScheduler} function
 * @returns Observable<number>
 */
export declare const retryInterval: (reconnectOptions: SocketReconnectOptions, scheduler: SocketReconnectScheduler) => Observable<number>;

/**
 * Row Data definition.
 * @public
 */
export declare type RowData = {
    ROW_REF: string;
};

/**
 * Schema Response definition.
 * @public
 */
export declare type SchemaResponse = {
    INBOUND: JSONSchema7;
    OUTBOUND: JSONSchema7;
    MESSAGE_TYPE: string;
    SOURCE_REF: string;
};

/**
 * A Serializer interface for serializing and deserializing objects.
 * @public
 */
export declare interface Serializer {
    /**
     * Serialize the given object into a string.
     * @param object - The object to serialize.
     * @returns The serialized string.
     * @deprecated - Use `serialize` from `JSONSerializer` from `@genesislcap/foundation-utils` instead.
     */
    serialize(object: any): string;
    /**
     * Deserialize a Response object into the expected type T.
     * @param response - The Response object to deserialize.
     * @returns A promise that resolves to the deserialized object of type T.
     * @deprecated - Use `deserialize` from `JSONSerializer` from `@genesislcap/foundation-utils` instead.
     */
    deserialize<T = any>(response: Response): Promise<T>;
    /**
     * Deserialize a MessageEvent object into the expected type T.
     * @param event - The MessageEvent object to deserialize.
     * @returns The deserialized object of type T.
     * @deprecated -Use `deserialize` from `JSONSerializer` from `@genesislcap/foundation-utils` instead.
     */
    deserialize<T = any>(event: MessageEvent): T;
    /**
     * Deserialize a string into the expected type T.
     * @param text - The string to deserialize.
     * @returns The deserialized object of type T.
     * @deprecated - Use `deserialize` from `JSONSerializer` from `@genesislcap/foundation-utils` instead.
     */
    deserialize<T = any>(text: string): T;
}

/**
 * The DI token for the Serializer interface.
 * @internal
 */
export declare const Serializer: InterfaceSymbol<Serializer>;

/**
 * Server Row definition.
 * @public
 */
export declare type ServerRow = {
    DETAILS: {
        OPERATION: 'INSERT' | 'MODIFY' | 'DELETE';
        ROW_REF: string;
    };
};

/**
 * Represents a session storage and its capabilities.
 * Provides methods to set and get items from the specified storage.
 * @public
 */
export declare interface Session {
    /**
     * The return URL for the session.
     * @public
     */
    returnUrl: string;
    /**
     * Sets the storage key prefix for the session.
     * @public
     * @param prefix - The prefix to set.
     */
    setStorageKeyPrefix(prefix: string): void;
    /**
     * Sets the authentication result for the session.
     * @public
     * @param info - The authentication result to set.
     */
    setAuthResult(info: Message): void;
    /**
     * Deletes the user from the session storage.
     * @public
     */
    deleteUserFromSessionStorage(): void;
    /**
     * Captures the return URL for the session.
     * @public
     */
    captureReturnUrl(): void;
    /**
     * Sets an item in the specified storage with the specified key and value.
     * @public
     * @param key - The key to set.
     * @param value - The value to set.
     * @param storage - The storage type to use.
     */
    setItem(key: string | UserInfoKeys, value: string, storage: StorageType): void;
    /**
     * Returns the value of the specified item in the specified storage.
     * @public
     * @param  key - The key to get.
     * @param storage - The storage type to use.
     * @returns - The value of the specified item.
     */
    getItem(key: string | UserInfoKeys, storage: StorageType): string;
    /**
     * Removes the item with the specified key from the specified storage.
     * @public
     * @param key - The key of the item to remove.
     * @param storage - The storage type to use.
     */
    removeItem(key: string | UserInfoKeys, storage: StorageType): void;
    /**
     * Sets an item in the local storage with the specified key and value.
     * @public
     * @param key - The key to set.
     * @param value - The value to set.
     */
    setLocalStorageItem(key: string | UserInfoKeys, value: string): void;
    /**
     * Sets an item in the session storage with the specified key and value.
     * @public
     * @param key - The key to set.
     * @param value - The value to set.
     */
    setSessionStorageItem(key: string | UserInfoKeys, value: string): void;
    /**
     * Returns the value of the item with the specified key from the local storage.
     * @public
     * @param key - The key of the item to get.
     * @returns - The value of the item with the specified key.
     */
    getLocalStorageItem(key: string | UserInfoKeys): string;
    /**
     * Returns the value of the item with the specified key from the session storage.
     * @public
     * @param key - The key of the item to get.
     * @returns - The value of the item with the specified key.
     */
    getSessionStorageItem(key: string | UserInfoKeys): string;
    /**
     * Removes the item with the specified key from the local storage.
     * @public
     * @param key - The key of the item to remove.
     */
    removeLocalStorageItem(key: string | UserInfoKeys): void;
    /**
     * Removes the item with the specified key from the session storage.
     * @public
     * @param key - The key of the item to remove.
     */
    removeSessionStorageItem(key: string | UserInfoKeys): void;
}

/**
 * The DI token for the Session interface.
 * @internal
 */
export declare const Session: InterfaceSymbol<Session>;

/**
 * @internal
 */
export declare class SessionMock implements Session {
    returnUrl: string;
    nextStorageItem: string;
    deleteUserFromSessionStorage(): void;
    setStorageKeyPrefix(prefix: string): void;
    setAuthResult(loginResult: Message): void;
    captureReturnUrl(): void;
    setItem(key: string, value: string): void;
    getItem(key: string): string;
    removeItem(key: string): void;
    setLocalStorageItem(key: string, value: string): void;
    setSessionStorageItem(key: string, value: string): void;
    getLocalStorageItem(key: string): string;
    getSessionStorageItem(key: string): string;
    removeLocalStorageItem(key: string): void;
    removeSessionStorageItem(key: string): void;
}

/**
 * Represents a WebSocket used to establish a communication channel between client and a Genesis server.
 * Provides methods for socket connection, sending and receiving WS messages.
 * @public
 */
export declare interface Socket {
    isConnected: boolean;
    isReconnecting: boolean;
    hasValidSession: boolean;
    isConnectedSubject: BehaviorSubject<boolean>;
    connect(host: string, options?: SocketConnectOptions, reconnectOptions?: SocketReconnectOptions): Promise<boolean>;
    /**
     * Disconnects the socket and clears all subscriptions
     */
    reset(): void;
    send<T>(message: Message<T | any>, needsHandling?: boolean): Promise<Message | any>;
    sendForStream<T>(message: Message<T | any>, onMessage: Function, onError: Function): Observable<Message>;
    socketMessages(): SocketSubject<Message>;
}

/**
 * The DI token for the Socket interface.
 * @internal
 */
export declare const Socket: InterfaceSymbol<Socket>;

/**
 * The Socket connection options definition.
 * @public
 */
export declare type SocketConnectOptions = {
    /**
     * Experimental flag to force reconnection all error scenarios (1011, 1012, etc). Won't rely on reconnectAttempts value.
     * @experimental
     * @remarks
     * This will force the socket to reconnect if the connection is lost even on "final" errors. Only disconnects when directly calling reset()
     * @see Socket#reset
     */
    alwaysOn?: boolean;
    /**
     * An observer to receive notifications of incoming messages from the server.
     */
    connectObserver?: Observer<Message>;
    /**
     * Enable sending heartbeat messages to the server to prevent the connection from timing out.
     */
    enableHeartbeat?: boolean;
    /**
     * Enable logging of heartbeat messages.
     */
    enableHeartbeatLogging?: boolean;
    /**
     * Enable logging of incoming and outgoing WS messages.
     */
    enableMessageLogging?: boolean;
    /**
     * Interval between heartbeat messages
     * @remarks
     * Time in milliseconds
     */
    heartbeatInterval?: number;
    /**
     * The path to a proxy server to use for the connection.
     */
    proxyPath?: string;
};

/**
 * The Socket message handler definition.
 * @public
 */
export declare class SocketMessageHandler {
    constructor(message: Message, onMessage: Function, onError: Function, isSubscription: boolean);
    /**
     * The message that was sent.
     * @public
     */
    get sentMessage(): Message;
    /**
     * The handler function for incoming messages.
     * @public
     */
    get onReceivedMessage(): Function;
    /**
     * The handler function for errors.
     * @public
     */
    get onReceivedError(): Function;
    /**
     * Whether the message handler is a subscription.
     * @public
     */
    get isSubscription(): boolean;
    /**
     * @internal
     */
    private _message;
    /**
     * @internal
     */
    private _onMessage;
    /**
     * @internal
     */
    private _onError;
    /**
     * @internal
     */
    private _isSubscription;
}

/**
 * @internal
 */
export declare class SocketMock implements Socket {
    hasValidSession: boolean;
    isConnected: boolean;
    isConnectedSubject: BehaviorSubject<boolean>;
    isReconnecting: boolean;
    nextMessage: Message;
    socketMessagesSubject: SocketSubject<Message<any>>;
    connect(host: string, options?: SocketConnectOptions, reconnectOptions?: SocketReconnectOptions): Promise<boolean>;
    disconnect(): void;
    send<T>(message: Message<any>): Promise<Message | any>;
    sendForStream<T>(message: Message<any>, onMessage: Function, onError: Function): Observable<Message>;
    socketMessages(): SocketSubject<Message>;
    reset(): void;
}

/**
 * Observable that wraps a WebSocketSubject and provides an Observable interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Observable.
 */
export declare class SocketObservable<T> extends Observable<T> {
    constructor(...args: any[]);
}

/**
 * Socket reconnection options
 * @public
 */
export declare type SocketReconnectOptions = {
    reconnectAttempts?: number;
    /**
     * Interval between reconnect attempts
     * @public
     * @remarks
     * Time in milliseconds
     */
    reconnectInterval?: number;
    /**
     * Observer to notify when reconnection occurs
     * @public
     */
    reconnectObserver?: Observer<any>;
    /**
     * Reconnect stream subscriptions on reconnection
     * @public
     * @remarks
     * Whether to re-use existing wsMessageHandlers and wsMessages instances
     */
    reconnectStreams?: boolean;
    /**
     * Strategy to be used on reconnection
     * @public
     */
    reconnectStrategy?: SocketReconnectStrategy;
};

/**
 * Socket reconnection scheduler
 * @public
 * @param reconnectOptions - {@link SocketReconnectOptions} options
 * @param attempt - number of attempts
 * @returns milliseconds to wait before next reconnect attempt
 */
export declare type SocketReconnectScheduler = (reconnectOptions: SocketReconnectOptions, attempts?: number) => number;

/**
 * Socket reconnection strategies
 * @public
 */
export declare enum SocketReconnectStrategy {
    Linear = "linear",
    Exponential = "exponential"
}

/**
 * Subject that wraps a WebSocketSubject and provides a Subject interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Subject.
 */
export declare class SocketSubject<T> extends Subject<T> {
    constructor();
}

export { SocketSubscription }

/**
 * Represents SSO authentication information.
 * @public
 */
export declare type SsoAuthInfo = {
    type: AuthType & AuthType.SSO;
    ssoToken: string;
};

/**
 * Represents SSO credentials for a user.
 * @public
 */
export declare type SSOCredentialsInput = {
    ssoToken: string;
};

/**
 * TODO: Review overlaps between user domain and general storage.
 * I think we should hand off to the credentialManager for some of these things.
 */
/**
 * The available browser storage types.
 * @public
 */
export declare type StorageType = 'local' | 'session';

/**
 * Converts the field definitions to field metadata.
 * @public
 * @param fieldDef - The field definitions.
 * @returns An array of field metadata.
 */
export declare const toFieldMetadata: (fieldDef: MetadataDetail[]) => FieldMetadata[];

/**
 * Types map for FieldTypeEnum.
 * @public
 */
export declare const TYPES_MAP: {
    STRING: FieldTypeEnum;
    ENUM: FieldTypeEnum;
    BOOLEAN: FieldTypeEnum;
    INT: FieldTypeEnum;
    LONG: FieldTypeEnum;
    DOUBLE: FieldTypeEnum;
    BIGDECIMAL: FieldTypeEnum;
    DATE: FieldTypeEnum;
    DATETIME: FieldTypeEnum;
};

/**
 * @internal
 */
export declare type UnknownUserPermissions = string;

/**
 * @internal
 */
export declare type UnknownUserProfiles = string;

/**
 * Polls a condition function until it returns true or a timeout is reached.
 * @public
 * @param conditionFunc - The condition function to poll.
 * @param timeoutMs - The timeout in milliseconds. Defaults to 300.
 * @returns A promise that resolves when the condition function returns true or rejects when the timeout is reached.
 */
export declare function until(conditionFunc: () => boolean, timeoutMs?: number): Promise<any>;

/**
 * Represents a User object with associated permissions and profiles.
 * @public
 */
export declare interface User extends UserOptions {
    hasAdminProfile(): boolean;
    hasPermission(permission: UserPermission | UserPermission[]): boolean;
    hasProfile(profile: UserProfile | UserProfile[]): boolean;
}

/**
 * The keys used to store user information in the session storage.
 * @internal
 */
declare type UserInfoKeys = 'authToken' | 'ssoToken' | 'refreshToken' | 'sessionId' | 'returnUrl' | 'hostUrl' | 'gridOptions';

/**
 * TODO: Evolve, re-locate to foundation-data, foundation-user or similar and add JSON schema based population,
 * validation, evolution, foundation-store interop etc.
 */
/**
 * Represents the core options for a user.
 * @internal
 */
declare interface UserOptions {
    readonly permissions: UserPermission[] | null;
    readonly profiles: UserProfile[] | null;
    readonly username: string | null;
}

/**
 * @internal
 */
export declare type UserPermission = CoreUserPermissions | UnknownUserPermissions;

/**
 * @internal
 */
export declare type UserProfile = CoreUserProfiles | UnknownUserProfiles;

/**
 * The DI token for the Connect interface (WS-only).
 * @internal
 */
export declare const WSConnect: InterfaceSymbol<Connect>;

export { }
